/***************************************************************
 * Name:      adresadialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include "adresadialog.h"
#include "dxutils.h"

AdresaDialog::AdresaDialog(bool edit, int id, QWidget *parent) :
    QDialog(parent), m_edit(edit), m_id(id)
{
    QDialogButtonBox *buttons;
    buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    QGridLayout *grid = new QGridLayout;
    grid->addWidget(new QLabel(QString::fromUtf8("Název")), 0, 0);
    m_nazev = new QLineEdit;
    grid->addWidget(m_nazev, 0, 1, 1, 3);
    grid->addWidget(new QLabel("Ulice"), 1, 0);
    m_ulice = new QLineEdit;
    grid->addWidget(m_ulice, 1, 1);
    grid->addWidget(new QLabel(QString::fromUtf8("čp")), 1, 2);
    m_cp = new QLineEdit;
    grid->addWidget(m_cp, 1, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("PSČ")), 2, 0);
    m_psc = new QLineEdit;
    grid->addWidget(m_psc, 2, 1);
    grid->addWidget(new QLabel(QString::fromUtf8("Místo")), 2, 2);
    m_misto = new QLineEdit;
    grid->addWidget(m_misto, 2, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("Země")), 3, 0);
    m_zeme = new QLineEdit;
    grid->addWidget(m_zeme, 3, 1, 1, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("IČ")), 4, 0);
    m_ic = new QLineEdit;
    grid->addWidget(m_ic, 4, 1, 1, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("DIČ")), 5, 0);
    m_dic = new QLineEdit;
    grid->addWidget(m_dic, 5, 1, 1, 3);
    grid->addWidget(new QLabel("E-mail"), 6, 0);
    m_email = new QLineEdit;
    grid->addWidget(m_email, 6, 1, 1, 3);
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addLayout(grid);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle("Kontakt");
    setupGUI();
}

void AdresaDialog::setupGUI()
{
    if(m_edit)
    {
        QSqlQuery q;
        q.exec(QString("SELECT nazev,ic,dic,ulice,cp,misto,psc,zeme,email FROM adresar_view WHERE id=%1").arg(m_id));
        q.first();
        m_nazev->setText(q.value(0).toString());
        m_ic->setText(q.value(1).toString());
        m_dic->setText(q.value(2).toString());
        m_ulice->setText(q.value(3).toString());
        m_cp->setText(q.value(4).toString());
        m_misto->setText(q.value(5).toString());
        m_psc->setText(q.value(6).toString());
        m_zeme->setText(q.value(7).toString());
        m_email->setText(q.value(8).toString());
    }
    else
    {
        m_zeme->setText(QString::fromUtf8("Česká republika"));
    }
}

bool AdresaDialog::checkNazev(const QString &nazev)
{
    return !nazev.isEmpty();
}

void AdresaDialog::accept()
{
    if(checkNazev(m_nazev->text()))
    {
        if(m_edit)
        {
            QSqlQuery q;
            q.prepare("UPDATE adresar_view SET nazev=?,ic=?,dic=?,ulice=?,cp=?,misto=?,"
                      "psc=?,zeme=?,email=? WHERE id=?");
            q.addBindValue(m_nazev->text());
            q.addBindValue(m_ic->text());
            q.addBindValue(m_dic->text());
            q.addBindValue(m_ulice->text());
            q.addBindValue(m_cp->text());
            q.addBindValue(m_misto->text());
            q.addBindValue(m_psc->text());
            q.addBindValue(m_zeme->text());
            q.addBindValue(m_email->text());
            q.addBindValue(m_id);
            if(!q.exec())
                QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
        }
        else
        {
            int id = dxutils::getLastID("adresa");
            m_id = dxutils::getLastID("adresar");
            QSqlQuery q;
            q.prepare("INSERT INTO adresa (id,ulice,cp,misto,psc,zeme) VALUES (?,?,?,?,?,?)");
            q.addBindValue(id);
            q.addBindValue(m_ulice->text());
            q.addBindValue(m_cp->text());
            q.addBindValue(m_misto->text());
            q.addBindValue(m_psc->text());
            q.addBindValue(m_zeme->text());
            if(!q.exec())
                QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
            q.prepare("INSERT INTO adresar (id,nazev,ic,dic,email,public,adresaid) VALUES (?,?,?,?,?,1,?)");
            q.addBindValue(m_id);
            q.addBindValue(m_nazev->text());
            q.addBindValue(m_ic->text());
            q.addBindValue(m_dic->text());
            q.addBindValue(m_email->text());
            q.addBindValue(id);
            if(!q.exec())
                QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
        }
        QDialog::accept();
    }
    else
    {
        QMessageBox::information(this, "Informace", QString::fromUtf8("Název nemůže být prázdný"), QMessageBox::Ok);
    }
}

void AdresaDialog::keyPressEvent(QKeyEvent *ev)
{
    if(!m_edit)
    {
        if(ev->key() != Qt::Key_Escape)
            QDialog::keyPressEvent(ev);
    }
    else
    {
        QDialog::keyPressEvent(ev);
    }
}
