/***************************************************************
 * Name:      adresardialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include "adresardialog.h"
#include "adresadialog.h"
#include "dxutils.h"

AdresarDialog::AdresarDialog(bool select, QWidget *parent) :
    QDialog(parent), m_selectedID(0), m_select(select)
{
    setupModel();
    QAction *add = new QAction(QIcon(":/icons/add.png"), QString::fromUtf8("Přidat"), this);
    add->setToolTip(QString::fromUtf8("Přidat kontakt"));
    connect(add, SIGNAL(triggered()), this, SLOT(slotAdd()));
    m_edit = new QAction(QIcon(":/icons/edit.png"), "Upravit", this);
    m_edit->setToolTip("Upravit kontakt");
    connect(m_edit, SIGNAL(triggered()), this, SLOT(slotEdit()));
    m_remove = new QAction(QIcon(":/icons/delete.png"), "Smazat", this);
    m_remove->setToolTip("Smazat kontakt");
    connect(m_remove, SIGNAL(triggered()), this, SLOT(slotRemove()));
    if(!m_model->rowCount())
    {
        m_edit->setEnabled(false);
        m_remove->setEnabled(false);
    }
    QToolBar *bar = new QToolBar("Firmy", this);
    bar->setStyleSheet("QToolBar { border: 0px }");
    bar->setMovable(false);
    bar->addAction(add);
    bar->addAction(m_edit);
    bar->addAction(m_remove);
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Cancel);
    if(m_select)
        buttons->addButton("Vybrat", QDialogButtonBox::AcceptRole);
    else
        buttons->addButton(QDialogButtonBox::Ok);
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    m_odberatele = new QTableView;
    m_odberatele->setModel(m_model);
    m_odberatele->hideColumn(0);
    m_odberatele->hideColumn(10);
    m_odberatele->horizontalHeader()->setStretchLastSection(true);
    m_odberatele->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_odberatele->setSelectionMode(QAbstractItemView::SingleSelection);
    m_odberatele->verticalHeader()->hide();
    m_odberatele->setContextMenuPolicy(Qt::CustomContextMenu);
    QFontMetrics fm = fontMetrics();
    m_odberatele->setColumnWidth(1, fm.width(" XXX Krakonosovo s.r.o. XXX "));
    connect(m_odberatele, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(slotDoubleclickOdberatele(QModelIndex)));
    connect(m_odberatele, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(slotPopup(QPoint)));
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(bar);
    mainLayout->addWidget(m_odberatele);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle(QString::fromUtf8("Adresář"));
}

void AdresarDialog::setupModel()
{
    m_model = new QSqlQueryModel(this);
    m_model->setQuery("SELECT * FROM adresar_view WHERE public = 1");
    m_model->setHeaderData(1, Qt::Horizontal, QString::fromUtf8("Název"));
    m_model->setHeaderData(2, Qt::Horizontal, QString::fromUtf8("IČ"));
    m_model->setHeaderData(3, Qt::Horizontal, QString::fromUtf8("DIČ"));
    m_model->setHeaderData(4, Qt::Horizontal, "E-mail");
    m_model->setHeaderData(5, Qt::Horizontal, "Ulice");
    m_model->setHeaderData(6, Qt::Horizontal, QString::fromUtf8("čp"));
    m_model->setHeaderData(7, Qt::Horizontal, QString::fromUtf8("Místo"));
    m_model->setHeaderData(8, Qt::Horizontal, QString::fromUtf8("PSČ"));
    m_model->setHeaderData(9, Qt::Horizontal, QString::fromUtf8("Země"));
}

void AdresarDialog::slotAdd()
{
    AdresaDialog *dialog = new AdresaDialog(false, 0, this);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        m_model->setQuery("SELECT * FROM adresar_view WHERE public = 1");
        if(m_model->rowCount())
        {
            m_edit->setEnabled(true);
            m_remove->setEnabled(true);
        }
    }
}

void AdresarDialog::slotEdit()
{
    QModelIndex current = m_odberatele->currentIndex();
    int id;
    if(current.isValid())
        id = m_model->record(current.row()).value("id").toInt();
    else
        id = m_model->record(0).value("id").toInt();
    AdresaDialog *dialog = new AdresaDialog(true, id, this);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        m_model->setQuery("SELECT * FROM adresar_view WHERE public = 1");
    }
}

void AdresarDialog::slotDoubleclickOdberatele(const QModelIndex &index)
{
    AdresaDialog *dialog = new AdresaDialog(true, m_model->record(index.row()).value("id").toInt(), this);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        m_model->setQuery("SELECT * FROM adresar_view WHERE public = 1");
    }
}

void AdresarDialog::slotRemove()
{
    QModelIndex current = m_odberatele->currentIndex();
    int id;
    QString name;
    if(current.isValid())
    {
        id = m_model->record(current.row()).value("id").toInt();
        name = m_model->record(current.row()).value("nazev").toString();
    }
    else
    {
        id = m_model->record(0).value("id").toInt();
        name = m_model->record(0).value("nazev").toString();
    }
    if(QMessageBox::question(this, "Dotaz", QString("Opravdu smazat adresu '%1'?").arg(name), QMessageBox::Ok|QMessageBox::Cancel) == QMessageBox::Ok)
    {
        QSqlQuery q(QString("SELECT adresaid FROM adresar WHERE id = %1").arg(id));
        q.first();
        int adresaid = q.value(0).toInt();
        q.prepare("DELETE FROM adresar WHERE id = ?");
        q.addBindValue(id);
        q.exec();
        m_model->setQuery("SELECT * FROM adresar_view WHERE public = 1");
        if(!m_model->rowCount())
        {
            m_edit->setEnabled(false);
            m_remove->setEnabled(false);
        }
        q.prepare("SELECT count() FROM adresar WHERE adresaid = %1");
        q.addBindValue(adresaid);
        q.exec();
        if(q.value(0).toInt() == 0)
        {
            q.prepare("DELETE FROM adresa WHERE id = ?");
            q.addBindValue(adresaid);
            q.exec();
        }
    }
}

void AdresarDialog::accept()
{
    QModelIndex current = m_odberatele->currentIndex();
    if(current.isValid())
        m_selectedID = m_model->record(current.row()).value("id").toInt();
    else
        m_selectedID = m_model->record(0).value("id").toInt();
    QDialog::accept();
}

int AdresarDialog::selectedID()
{
    return m_selectedID;
}

void AdresarDialog::slotPopup(const QPoint &pos)
{
    QModelIndex current = m_odberatele->currentIndex();
    if(current.isValid())
    {
        QMenu popup;
        popup.addAction(m_edit);
        popup.addAction(m_remove);
        popup.exec(m_odberatele->mapToGlobal(pos));
    }
}

