/***************************************************************
 * Name:      editordialog.h
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef EDITORDIALOG_H
#define EDITORDIALOG_H

#include <QtGui>
#if (QT_VERSION > 0x050000)
#include <QtWidgets>
#endif

class TemplateDialog : public QDialog
{
    Q_OBJECT
public:
    explicit TemplateDialog(QWidget *parent = 0);

    QString templateName() const;
private:
    QComboBox *m_template;
};

class TemplateHighlighter : public QSyntaxHighlighter
{
    Q_OBJECT
public:
    TemplateHighlighter(QTextDocument *parent = 0);
protected:
    void highlightBlock(const QString &text);
private:
    QStringList m_keywords;
    QTextCharFormat m_keywordFormat;
    QTextCharFormat m_tagFormat;
};

class EditorDialog : public QMainWindow
{
    Q_OBJECT
public:
    explicit EditorDialog(QWidget *parent = 0);
protected:
    void closeEvent(QCloseEvent *event);
private:
    QTextEdit *m_edit;
    TemplateHighlighter *m_highlighter;
    QString m_filename;
    QAction *m_open;
    QAction *m_save;
    QAction *m_undo;
    QAction *m_redo;
    QAction *m_cut;
    QAction *m_copy;
    QAction *m_paste;
    QAction *m_print;
    QAction *m_preview;
    QAction *m_find;
    QAction *m_replace;

    void createActions();
    void createMenu();
    void createToolbar();
    void setFileName(const QString &filename);
    void setTitle();
    bool saveQuestion();
    void save();
    void saveAs();
    void saveFile(const QString &filename);
private slots:
    void slotDocumentChanged();
    void slotCmdOpen();
    void slotCmdSave();
    void slotCmdPrint();
    void slotCmdPreview();
    void slotCmdFind();
    void slotCmdFindReplace();
};

#endif // EDITORDIALOG_H
