/***************************************************************
 * Name:      emailfadialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <QPrinter>
#include <QPrintDialog>
#include "emailfadialog.h"
#include "utils/faparser.h"
#include "dxutils.h"
#include "preferences.h"

EmailfaDialog::EmailfaDialog(int id, QWidget *parent) :
    QDialog(parent), m_id(id)
{
    QSqlQuery faktura(QString("SELECT platce FROM faktura WHERE id = %1").arg(m_id));
    faktura.first();
    m_platce = faktura.value(0).toBool();
    QHBoxLayout *print = new QHBoxLayout;
    m_internal = new QPushButton(QString::fromUtf8("Použít výchozí šablonu"));
    connect(m_internal, SIGNAL(clicked()), this, SLOT(slotInternal()));
    print->addWidget(m_internal);
    m_custom = new QPushButton(QString::fromUtf8("Použít vlastní šablonu"));
    connect(m_custom, SIGNAL(clicked()), this, SLOT(slotCustom()));
    print->addWidget(m_custom);
    m_email = new QPushButton("&Odeslat");
    connect(m_email, SIGNAL(clicked()), this, SLOT(slotEmail()));
    print->addWidget(m_email);
    print->addStretch();
    QDialogButtonBox *buttons;
    buttons = new QDialogButtonBox(QDialogButtonBox::Ok);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    m_faktura = new QWebView;
    createText();
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addLayout(print);
    mainLayout->addWidget(m_faktura);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle("Odeslat fakturu");
}

void EmailfaDialog::slotEmail()
{
    QString path = QDir::tempPath();
#ifdef Q_OS_WIN
    path = QProcessEnvironment::systemEnvironment().value("APPDATA")+"/dxfaktura";
#else
#if (QT_VERSION > 0x050000)
    path = QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0)+"/.dxfaktura";
#else
    path = QDesktopServices::storageLocation(QDesktopServices::HomeLocation)+"/.dxfaktura";
#endif
#endif
    QPrinter printer;
    printer.setOutputFormat(QPrinter::PdfFormat);
    QSqlQuery faktura(QString("SELECT * FROM faktura WHERE id = %1").arg(m_id));
    faktura.first();
    path.append("/"+faktura.value(faktura.record().indexOf("cislo")).toString()+".pdf");
    int adresarid = faktura.value(faktura.record().indexOf("adresarid")).toInt();
    printer.setOutputFileName(path);
    m_faktura->print(&printer);
    faktura.exec(QString("SELECT email FROM adresar_view WHERE id = %1").arg(adresarid));
    faktura.first();
    SendDialog dialog(path, faktura.value(0).toString(), this);
    dialog.setModal(true);
    dialog.exec();
    QDir dir;
    dir.remove(path);
}

void EmailfaDialog::createText()
{
    QString filename = selectTemplate();
    QFile file(filename);
    if(!file.open(QFile::ReadOnly | QFile::Text))
    {
        m_faktura->setHtml(QString::fromUtf8("<html><head></head><body>Nemohu otevřít šablonu: %1</body></html>").arg(filename));
        return;
    }
    QApplication::setOverrideCursor(Qt::WaitCursor);
    FaParser parser(m_id);
    m_faktura->setHtml(parser.parseFile(filename));
    QApplication::restoreOverrideCursor();
}

QString EmailfaDialog::selectTemplate()
{
    QString temp = "";
    QSqlQuery q(QString("SELECT vydana,typid FROM faktura WHERE id=%1").arg(m_id));
    q.first();
    if(q.value(0).toBool())
    {
        switch(q.value(1).toInt())
        {
            case 1:
            {
                if(m_platce)
                {
                    temp = preferences.m_favydanaplatce;
                    if(temp == "internal")
                    {
                        m_internal->hide();
                        return ":/templates/favydanaplatce.html";
                    }
                    else return temp;
                }
                else
                {
                    temp = preferences.m_favydananeplatce;
                    if(temp == "internal")
                    {
                        m_internal->hide();
                        return ":/templates/favydananeplatce.html";
                    }
                    else return temp;
                }
            }
            default:
            {
                if(m_platce)
                {
                    temp = preferences.m_zalohavydanaplatce;
                    if(temp == "internal")
                    {
                        m_internal->hide();
                        return ":/templates/zalohavydanaplatce.html";
                    }
                    else return temp;
                }
                else
                {
                    temp = preferences.m_zalohavydananeplatce;
                    if(temp == "internal")
                    {
                        m_internal->hide();
                        return ":/templates/zalohavydananeplatce.html";
                    }
                    else return temp;
                }
            }
        }
    }
    else
    {
        switch(q.value(1).toInt())
        {
            case 1:
            {
                if(m_platce)
                {
                    temp = preferences.m_faprijataplatce;
                    if(temp == "internal")
                    {
                        m_internal->hide();
                        return ":/templates/faprijataplatce.html";
                    }
                    else return temp;
                }
                else
                {
                    temp = preferences.m_faprijataneplatce;
                    if(temp == "internal")
                    {
                        m_internal->hide();
                        return ":/templates/faprijataneplatce.html";
                    }
                    else return temp;
                }
            }
            default:
            {
                if(m_platce)
                {
                    temp = preferences.m_zalohaprijataplatce;
                    if(temp == "internal")
                    {
                        m_internal->hide();
                        return ":/templates/zalohaprijataplatce.html";
                    }
                    else return temp;
                }
                else
                {
                    temp = preferences.m_zalohaprijataneplatce;
                    if(temp == "internal")
                    {
                        m_internal->hide();
                        return ":/templates/zalohaprijataneplatce.html";
                    }
                    else return temp;
                }
            }
        }
    }
    return temp;
}

void EmailfaDialog::slotInternal()
{
    QSqlQuery q(QString("SELECT vydana,typid FROM faktura WHERE id=%1").arg(m_id));
    q.first();
    if(q.value(0).toBool())
    {
        switch(q.value(1).toInt())
        {
            case 1:
            {
                if(m_platce)
                {
                    preferences.m_favydanaplatce = "internal";
                }
                else
                {
                    preferences.m_favydananeplatce = "internal";
                }
            }
            default:
            {
                if(m_platce)
                {
                    preferences.m_zalohavydanaplatce = "internal";
                }
                else
                {
                    preferences.m_zalohavydananeplatce = "internal";
                }
            }
        }
    }
    else
    {
        switch(q.value(1).toInt())
        {
            case 1:
            {
                if(m_platce)
                {
                    preferences.m_faprijataplatce = "internal";
                }
                else
                {
                    preferences.m_faprijataneplatce = "internal";
                }
            }
            default:
            {
                if(m_platce)
                {
                    preferences.m_zalohaprijataplatce = "internal";
                }
                else
                {
                    preferences.m_zalohaprijataneplatce = "internal";
                }
            }
        }
    }
    m_internal->hide();
    createText();
    updateSettings();
}

void EmailfaDialog::slotCustom()
{
#if (QT_VERSION > 0x050000)
    QString filename = QFileDialog::getOpenFileName(this, QString::fromUtf8("Vyberte šablonu"), QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0), QString::fromUtf8("Šablona (*htm *html)"));
#else
    QString filename = QFileDialog::getOpenFileName(this, QString::fromUtf8("Vyberte šablonu"), QDesktopServices::storageLocation(QDesktopServices::HomeLocation), QString::fromUtf8("Šablona (*htm *html)"));
#endif
    if(!filename.isEmpty())
    {
        QSqlQuery q(QString("SELECT vydana,typid FROM faktura WHERE id=%1").arg(m_id));
        q.first();
        if(q.value(0).toBool())
        {
            switch(q.value(1).toInt())
            {
                case 1:
                {
                    if(m_platce)
                    {
                        preferences.m_favydanaplatce = filename;
                    }
                    else
                    {
                        preferences.m_favydananeplatce = filename;
                    }
                }
                default:
                {
                    if(m_platce)
                    {
                        preferences.m_zalohavydanaplatce = filename;
                    }
                    else
                    {
                        preferences.m_zalohavydananeplatce = filename;
                    }
                }
            }
        }
        else
        {
            switch(q.value(1).toInt())
            {
                case 1:
                {
                    if(m_platce)
                    {
                        preferences.m_faprijataplatce = filename;
                    }
                    else
                    {
                        preferences.m_faprijataneplatce = filename;
                    }
                }
                default:
                {
                    if(m_platce)
                    {
                        preferences.m_zalohaprijataplatce = filename;
                    }
                    else
                    {
                        preferences.m_zalohaprijataneplatce = filename;
                    }
                }
            }
        }
    }
    m_internal->show();
    createText();
    updateSettings();
}

void EmailfaDialog::updateSettings()
{
    QSqlQuery q;
    q.prepare("UPDATE settings SET favydanaplatce=?,favydananeplatce=?,zalohavydanaplatce=?,zalohavydananeplatce=?,"
              "faprijataplatce=?,faprijataneplatce=?,zalohaprijataplatce=?,zalohaprijataneplatce=? WHERE firmaid=?");
    q.addBindValue(preferences.m_favydanaplatce);
    q.addBindValue(preferences.m_favydananeplatce);
    q.addBindValue(preferences.m_zalohavydanaplatce);
    q.addBindValue(preferences.m_zalohavydananeplatce);
    q.addBindValue(preferences.m_faprijataplatce);
    q.addBindValue(preferences.m_faprijataneplatce);
    q.addBindValue(preferences.m_zalohaprijataplatce);
    q.addBindValue(preferences.m_zalohaprijataneplatce);
    q.addBindValue(preferences.m_firmaid);
    if(!q.exec())
        QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
}



SendDialog::SendDialog(const QString &file, const QString &recipient, QWidget *parent) :
    QDialog(parent), m_file(file)
{
    QVBoxLayout *mainLayout = new QVBoxLayout;
    QGridLayout *grid = new QGridLayout;
    grid->addWidget(new QLabel("SMTP server"),0,0);
    m_server = new QLineEdit;
    grid->addWidget(m_server,0,1);
    grid->addWidget(new QLabel("Port"),1,0);
    m_port = new QSpinBox;
    m_port->setRange(0,65535);
    grid->addWidget(m_port,1,1);
    grid->addWidget(new QLabel("SSL"),2,0);
    m_ssl = new QCheckBox;
    grid->addWidget(m_ssl,2,1);
    grid->addWidget(new QLabel(QString::fromUtf8("Použít autentifikaci")),3,0);
    m_autentifikace = new QCheckBox;
    grid->addWidget(m_autentifikace,3,1);
    grid->addWidget(new QLabel(QString::fromUtf8("Uživatelské jméno")),4,0);
    m_username = new QLineEdit;
    grid->addWidget(m_username,4,1);
    grid->addWidget(new QLabel("Heslo"),5,0);
    m_password = new QLineEdit;
    m_password->setEchoMode(QLineEdit::PasswordEchoOnEdit);
    grid->addWidget(m_password,5,1);
    grid->addWidget(new QLabel(QString::fromUtf8("Odesílatel")),6,0);
    m_sender = new QLineEdit;
    grid->addWidget(m_sender,6,1);
    grid->addWidget(new QLabel(QString::fromUtf8("Příjemce")),7,0);
    m_recipient = new QLineEdit;
    m_recipient->setText(recipient);
    grid->addWidget(m_recipient,7,1);
    grid->addWidget(new QLabel(QString::fromUtf8("Předmět")),8,0);
    m_subject = new QLineEdit;
    grid->addWidget(m_subject,8,1);
    grid->addWidget(new QLabel("Text"),9,0);
    m_text = new QTextEdit;
    m_text->setMaximumHeight(150);
    grid->addWidget(m_text,9,1);

    m_server->setText(preferences.m_server);
    m_port->setValue(preferences.m_port);
    m_ssl->setChecked(preferences.m_ssl);
    m_autentifikace->setChecked(preferences.m_autentifikace);
    m_username->setText(preferences.m_username);
    m_password->setText(preferences.m_password);
    m_sender->setText(preferences.m_sender);
    m_subject->setText(preferences.m_subject);
    m_text->setPlainText(preferences.m_text);

    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Cancel);
    buttons->addButton("Odeslat", QDialogButtonBox::AcceptRole);
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    mainLayout->addLayout(grid);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle("Odeslat fakturu");
}

EmailAddress *SendDialog::stringToEmail(const QString &str)
{
    int p1 = str.indexOf("<");
    int p2 = str.indexOf(">");
    if (p1 == -1)
    {
        // no name, only email address
        return new EmailAddress(str);
    }
    else
    {
        return new EmailAddress(str.mid(p1 + 1, p2 - p1 - 1), str.left(p1));
    }
}

void SendDialog::errorMessage(const QString &message)
{
    QErrorMessage err(this);
    err.showMessage(message);
    err.exec();
}

void SendDialog::accept()
{
    QStringList rcptStringList = m_recipient->text().split(';');
    SmtpClient smtp(m_server->text(), m_port->value(), m_ssl->isChecked() ? SmtpClient::SslConnection : SmtpClient::TcpConnection);
    MimeMessage message;
    message.setSender(stringToEmail(m_sender->text()));
    message.setSubject(m_subject->text());
    for(int i=0; i<rcptStringList.size(); ++i)
         message.addRecipient(stringToEmail(rcptStringList.at(i)));
    MimeHtml html;
    html.setHtml(m_text->toHtml());
    message.addPart(&html);
    message.addPart(new MimeAttachment(new QFile(m_file)));
    if(!smtp.connectToHost())
    {
        errorMessage(QString::fromUtf8("Připojení selhalo"));
        return;
    }
    if(m_autentifikace->isChecked())
    {
        if(!smtp.login(m_username->text(), m_password->text()))
        {
            errorMessage("Autentifikace selhala");
            return;
        }
    }
    if(!smtp.sendMail(message))
    {
        errorMessage(QString::fromUtf8("Odeslání e-mailu selhalo"));
        return;
    }
    else
    {
        QMessageBox okMessage (this);
        okMessage.setText(QString::fromUtf8("E-mail úspěšně odeslán"));
        okMessage.exec();
    }
    smtp.quit();
    QDialog::accept();
}
