/***************************************************************
 * Name:      fakturadialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include "fakturadialog.h"
#include "dxutils.h"
#include "adresardialog.h"
#include "bankadialog.h"
#include "fauhradadialog.h"
#include "polozkadialog.h"
#include "preferences.h"

//pro pridani
FakturaDialog::FakturaDialog(int typ, bool vydana, QWidget *parent) :
    QDialog(parent), m_edit(false), m_vydana(vydana), m_idAdresa(0), m_typFaktury(typ)
{
    m_id = dxutils::getLastID("faktura");
    createGUI();
}

//pro editaci
FakturaDialog::FakturaDialog(int id, QWidget *parent) :
    QDialog(parent), m_edit(true), m_id(id)
{
    QSqlQuery q(QString("SELECT vydana,adresarid FROM faktura WHERE id=%1").arg(id));
    q.first();
    m_vydana = q.value(0).toBool();
    m_idAdresa = q.value(1).toInt();
    q.exec(QString("SELECT typid FROM faktura WHERE id=%1").arg(id));
    q.first();
    m_typFaktury = q.value(0).toInt();
    createGUI();
}

void FakturaDialog::createGUI()
{
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    QHBoxLayout *layout1 = new QHBoxLayout;
    layout1->addLayout(createFakturaGrid());
    layout1->addLayout(createAdresaBox());
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addLayout(layout1);
    m_dph = new QWidget;
    m_dphGrid = new QGridLayout;
    m_dphGrid->addWidget(new QLabel("Sazba DPH"), 0, 0);
    m_dphGrid->addWidget(new QLabel(QString::fromUtf8("Základ")), 0, 1);
    m_dphGrid->addWidget(new QLabel("DPH"), 0, 2);
    m_dphGrid->addWidget(new QLabel(QString::fromUtf8("Celkem vč. DPH")), 0, 3);
    mainLayout->addWidget(m_dph);
    m_dph->setLayout(m_dphGrid);
    if(!isPlatce()) m_dph->hide();
    mainLayout->addWidget(createPolozkaBox());
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle("Faktura");
    updateCelkem();
    if(isPlatce()) updateDPH();
}

QGridLayout *FakturaDialog::createFakturaGrid()
{
    QSqlQuery q;
    QGridLayout *grid = new QGridLayout;
    grid->addWidget(new QLabel("Popis"), 0, 0);
    m_popis = new QLineEdit;
    grid->addWidget(m_popis, 0, 1, 1, 2);
    grid->addWidget(new QLabel(QString::fromUtf8("Číslo")), 1, 0);
    m_cislo = new QLineEdit;
    grid->addWidget(m_cislo, 1, 1, 1, 2);
    grid->addWidget(new QLabel(QString::fromUtf8("Variabilní symbol")), 2, 0);
    m_vs = new QLineEdit;
    grid->addWidget(m_vs, 2, 1, 1, 2);
    grid->addWidget(new QLabel(QString::fromUtf8("Konstantní symbol")), 3, 0);
    m_ks = new QLineEdit;
    grid->addWidget(m_ks, 3, 1, 1, 2);
    grid->addWidget(new QLabel(QString::fromUtf8("Forma úhrady")), 4, 0);
    m_uhrada = new QComboBox;
    connect(m_uhrada, SIGNAL(currentIndexChanged(int)), this, SLOT(slotUhrada(int)));
    grid->addWidget(m_uhrada, 4, 1);
    m_uhradaAdd = new QPushButton("...");
    connect(m_uhradaAdd, SIGNAL(clicked()), this, SLOT(slotAddUhrada()));
    grid->addWidget(m_uhradaAdd, 4, 2);
    m_labelBanka = new QLabel("Banka");
    grid->addWidget(m_labelBanka, 5, 0);
    m_banka = new QComboBox;
    grid->addWidget(m_banka, 5, 1);
    m_bankaAdd = new QPushButton("...");
    connect(m_bankaAdd, SIGNAL(clicked()), this, SLOT(slotAddBanka()));
    grid->addWidget(m_bankaAdd, 5, 2);
    if(!m_vydana)
    {
        m_labelBanka->hide();
        m_banka->hide();
        m_bankaAdd->hide();
    }
    grid->addWidget(new QLabel(QString::fromUtf8("Datum vystavení")), 6, 0);
    m_vystaveni = new QDateEdit(getDate("vystaveni"));
    m_vystaveni->setCalendarPopup(true);
    grid->addWidget(m_vystaveni, 6, 1, 1, 2);
    m_labelDUZP = new QLabel(QString::fromUtf8("Datum usk.zdan.plnění"));
    grid->addWidget(m_labelDUZP, 7, 0);
    m_duzp = new QDateEdit(getDate("duzp"));
    m_duzp->setCalendarPopup(true);
    grid->addWidget(m_duzp, 7, 1, 1, 2);
    if(!isPlatce())
    {
        m_labelDUZP->hide();
        m_duzp->hide();
    }
    grid->addWidget(new QLabel("Datum splatnosti"), 8, 0);
    m_splatnost = new QDateEdit(getDate("splatnost"));
    m_splatnost->setCalendarPopup(true);
    grid->addWidget(m_splatnost, 8, 1, 1, 2);
    grid->addWidget(new QLabel(QString::fromUtf8("Měna")), 9, 0);
    m_mena = new QComboBox;
    grid->addWidget(m_mena, 9, 1);
    m_menaAdd = new QPushButton("...");
    connect(m_menaAdd, SIGNAL(clicked()), this, SLOT(slotAddMena()));
    grid->addWidget(m_menaAdd, 9, 2);
    grid->addWidget(new QLabel("<b>Celkem</b>"), 10, 0);
    m_celkem = new QLabel;
    m_celkem->setFrameStyle(QFrame::Sunken|QFrame::Panel);
    m_celkem->setAlignment(Qt::AlignRight);
    grid->addWidget(m_celkem, 10, 1, 1, 2);
    if(m_edit)
    {
        q.exec(QString("SELECT popis,cislo,vs,ks FROM faktura WHERE id=%1").arg(m_id));
        q.first();
        m_popis->setText(q.value(0).toString());
        m_cislo->setText(q.value(1).toString());
        m_vs->setText(q.value(2).toString());
        m_ks->setText(q.value(3).toString());
    }
    else
    {
        m_idAdresa = 0;
        QDate current = QDate::currentDate();
        QDate splatnost = current.addDays(preferences.m_splatnost);
        QSqlQuery q;
        m_id = dxutils::getLastID("faktura");
        QString last = QString::number(getLastFaktura());
        QString prefix = m_vydana?preferences.m_prefixVydana:preferences.m_prefixPrijata;
        QString cislo;
        if((prefix.length()+last.length())<10) cislo = QString("%1%2").arg(prefix).arg(last,10-prefix.length(),QChar('0'));
        else cislo = QString("%1%2").arg(prefix).arg(last);
        QString vs = QString("%1%2").arg(preferences.m_rok).arg(last,6,QChar('0'));
        q.prepare("INSERT INTO faktura (id,firmaid,cislo,vs,last,typid,uhradaid,bankaid,menaid,vydana,platce,vystaveni,duzp,splatnost,rok)"
                  " VALUES (?,?,?,?,?,?,1,0,1,?,?,?,?,?,?)");
        q.addBindValue(m_id);
        q.addBindValue(preferences.m_firmaid);
        q.addBindValue(cislo);
        q.addBindValue(vs);
        q.addBindValue(last);
        q.addBindValue(m_typFaktury);
        q.addBindValue(m_vydana?1:0);
        q.addBindValue(isPlatce());
        q.addBindValue(current.toString("dd.MM.yyyy"));
        q.addBindValue(current.toString("dd.MM.yyyy"));
        q.addBindValue(splatnost.toString("dd.MM.yyyy"));
        q.addBindValue(preferences.m_rok);
        if(!q.exec()) QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
        m_cislo->setText(cislo);
        m_vs->setText(vs);
    }
    int uhrada = getUhrada();
    int i = 0;
    q.exec("SELECT id,uhrada FROM fauhrada");
    while(q.next())
    {
        m_uhrada->addItem(q.value(1).toString(),q.value(0).toInt());
        if(uhrada == q.value(0).toInt())
            m_uhrada->setCurrentIndex(i);
        i++;
    }
    if(m_uhrada->count() == 1) m_uhrada->setCurrentIndex(0);
    int banka = getBanka();
    i = 0;
    q.exec("SELECT id,cislo,banka FROM banka WHERE id <> 0");
    while(q.next())
    {
        m_banka->addItem(QString("%1/%2").arg(q.value(1).toString()).arg(q.value(2).toString()),q.value(0).toInt());
        if(banka == q.value(0).toInt())
            m_banka->setCurrentIndex(i);
        i++;
    }
    if(m_banka->count() == 1) m_banka->setCurrentIndex(0);
    int mena = getMena();
    i = 0;
    q.exec("SELECT id,zkratka FROM mena");
    while(q.next())
    {
        m_mena->addItem(q.value(1).toString(),q.value(0).toInt());
        if(mena == q.value(0).toInt())
            m_mena->setCurrentIndex(i);
        i++;
    }
    if(m_mena->count() == 1) m_mena->setCurrentIndex(0);
    connect(m_vystaveni, SIGNAL(dateChanged(QDate)), this, SLOT(slotVystaveniChanged(QDate)));
    return grid;
}

QVBoxLayout *FakturaDialog::createAdresaBox()
{
    QVBoxLayout *vlayout = new QVBoxLayout;
    int frameStyle = QFrame::Sunken|QFrame::Panel;
    QGroupBox *box = new QGroupBox(m_vydana?QString::fromUtf8("Odběratel"):"Dodavatel");
    QHBoxLayout *hlayout = new QHBoxLayout;
    QGridLayout *grid = new QGridLayout;
    grid->addWidget(new QLabel(QString::fromUtf8("Název")), 0, 0);
    m_nazev = new QLabel;
    m_nazev->setFrameStyle(frameStyle);
    grid->addWidget(m_nazev, 0, 1, 1, 3);
    grid->addWidget(new QLabel("Ulice"), 1, 0);
    m_ulice = new QLabel;
    m_ulice->setFrameStyle(frameStyle);
    grid->addWidget(m_ulice, 1, 1);
    grid->addWidget(new QLabel(QString::fromUtf8("čp")), 1, 2);
    m_cp = new QLabel;
    m_cp->setFrameStyle(frameStyle);
    grid->addWidget(m_cp, 1, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("PSČ")), 2, 0);
    m_psc = new QLabel;
    m_psc->setFrameStyle(frameStyle);
    grid->addWidget(m_psc, 2, 1);
    grid->addWidget(new QLabel(QString::fromUtf8("Místo")), 2, 2);
    m_misto = new QLabel;
    m_misto->setFrameStyle(frameStyle);
    grid->addWidget(m_misto, 2, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("Země")), 3, 0);
    m_zeme = new QLabel;
    m_zeme->setFrameStyle(frameStyle);
    grid->addWidget(m_zeme, 3, 1, 1, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("IČ")), 4, 0);
    m_ic = new QLabel;
    m_ic->setFrameStyle(frameStyle);
    grid->addWidget(m_ic, 4, 1, 1, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("DIČ")), 5, 0);
    m_dic = new QLabel;
    m_dic->setFrameStyle(frameStyle);
    grid->addWidget(m_dic, 5, 1, 1, 3);
    hlayout->addLayout(grid);
    QVBoxLayout *buttons = new QVBoxLayout;
    m_selectAdresa = new QPushButton("...");
    m_selectAdresa->setToolTip(m_vydana?QString::fromUtf8("Vyberte odběratele"):"Vyberte dodavatele");
    connect(m_selectAdresa, SIGNAL(clicked()), this, SLOT(slotSelectAdresa()));
    buttons->addWidget(m_selectAdresa);
    m_clearAdresa = new QPushButton(QIcon(":/icons/clear.png"), "");
    m_clearAdresa->setToolTip(m_vydana?QString::fromUtf8("Smazat odběratele"):"Smazat dodavatele");
    connect(m_clearAdresa, SIGNAL(clicked()), this, SLOT(slotClearAdresa()));
    m_clearAdresa->setEnabled(false);
    buttons->addWidget(m_clearAdresa);
    buttons->addStretch();
    hlayout->addLayout(buttons);
    box->setLayout(hlayout);
    vlayout->addWidget(box);
    vlayout->addStretch();
    fillAdresaBox();
    return vlayout;
}

QGroupBox *FakturaDialog::createPolozkaBox()
{
    m_modelPolozka = new PolozkaModel(this);
    m_modelPolozka->setQuery(QString("SELECT id,popis,pocet,mj,cenamj,celkem FROM polozka WHERE fakturaid = %1").arg(m_id));
    m_modelPolozka->setHeaderData(1, Qt::Horizontal, "Popis");
    m_modelPolozka->setHeaderData(2, Qt::Horizontal, QString::fromUtf8("Množství"));
    m_modelPolozka->setHeaderData(4, Qt::Horizontal, "Cena za mj");
    m_modelPolozka->setHeaderData(5, Qt::Horizontal, "Cena celkem");
    QGroupBox *box = new QGroupBox(QString::fromUtf8("Položky"));
    QVBoxLayout *vlayout = new QVBoxLayout;
    QAction *addPolozka = new QAction(QIcon(":/icons/add.png"), QString::fromUtf8("Přidat"), this);
    addPolozka->setToolTip(QString::fromUtf8("Přidat položku"));
    connect(addPolozka, SIGNAL(triggered()), this, SLOT(slotAddPolozka()));
    m_editPolozka = new QAction(QIcon(":/icons/edit.png"), QString::fromUtf8("Upravit"), this);
    m_editPolozka->setToolTip(QString::fromUtf8("Upravit položku"));
    connect(m_editPolozka, SIGNAL(triggered()), this, SLOT(slotEditPolozka()));
    m_removePolozka = new QAction(QIcon(":/icons/delete.png"), QString::fromUtf8("Smazat"), this);
    m_removePolozka->setToolTip(QString::fromUtf8("Smazat položku"));
    connect(m_removePolozka, SIGNAL(triggered()), this, SLOT(slotRemovePolozka()));
    if(!m_modelPolozka->rowCount())
    {
        m_editPolozka->setEnabled(false);
        m_removePolozka->setEnabled(false);
    }
    QToolBar *bar = new QToolBar("Polozky", this);
    bar->setStyleSheet("QToolBar { border: 0px }");
    bar->setMovable(false);
    bar->addAction(addPolozka);
    bar->addAction(m_editPolozka);
    bar->addAction(m_removePolozka);
    m_polozky = new QTableView;
    m_polozky->setModel(m_modelPolozka);
    m_polozky->hideColumn(0);
    m_polozky->horizontalHeader()->setStretchLastSection(true);
    m_polozky->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_polozky->setSelectionMode(QAbstractItemView::SingleSelection);
    m_polozky->verticalHeader()->hide();
    m_polozky->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(m_polozky, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(slotDoubleclickPolozky(QModelIndex)));
    connect(m_polozky, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(slotPopup(QPoint)));
    vlayout->addWidget(bar);
    vlayout->addWidget(m_polozky);
    box->setLayout(vlayout);
    return box;
}

void FakturaDialog::fillAdresaBox()
{
    QSqlQuery q(QString("SELECT nazev,ic,dic,ulice,cp,misto,psc,zeme FROM adresar_view WHERE id=%1").arg(m_idAdresa));
    q.first();
    m_nazev->setText(q.value(0).toString());
    m_ic->setText(q.value(1).toString());
    m_dic->setText(q.value(2).toString());
    m_ulice->setText(q.value(3).toString());
    m_cp->setText(q.value(4).toString());
    m_misto->setText(q.value(5).toString());
    m_psc->setText(q.value(6).toString());
    m_zeme->setText(q.value(7).toString());
    if(!q.value(0).toString().isEmpty())
        m_clearAdresa->setEnabled(true);
    else
        m_clearAdresa->setEnabled(false);
}

void FakturaDialog::accept()
{
    QSqlQuery q;
    q.exec(QString("SELECT SUM(celkem) FROM polozka WHERE fakturaid = %1").arg(m_id));
    q.first();
    float celkem = q.value(0).toFloat();
    q.prepare("UPDATE faktura SET popis=?,adresarid=?,vystaveni=?,splatnost=?,bankaid=?,"
              "uhradaid=?,menaid=?,celkem=? WHERE id=?");
    q.addBindValue(m_popis->text());
    q.addBindValue(m_idAdresa);
    q.addBindValue(m_vystaveni->date().toString("dd.MM.yyyy"));
    q.addBindValue(m_splatnost->date().toString("dd.MM.yyyy"));
    q.addBindValue(m_banka->count() && !isUhradaPokladnou() ? m_banka->itemData(m_banka->currentIndex()) : 0);
    q.addBindValue(m_uhrada->itemData(m_uhrada->currentIndex()));
    q.addBindValue(m_mena->itemData(m_mena->currentIndex()));
    q.addBindValue(celkem);
    q.addBindValue(m_id);
    if(!q.exec())
        QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
    QDialog::accept();
}

void FakturaDialog::reject()
{
    if(!m_edit)
    {
        QSqlQuery q;
        q.exec(QString("DELETE FROM faktura WHERE id=%1").arg(m_id));
    }
    QDialog::reject();
}

void FakturaDialog::slotSelectAdresa()
{
    AdresarDialog *dialog = new AdresarDialog(true, this);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        m_idAdresa = dialog->selectedID();
        fillAdresaBox();
    }
}

void FakturaDialog::slotClearAdresa()
{
    m_idAdresa = 0;
    fillAdresaBox();
}

int FakturaDialog::getLastFaktura()
{
    QSqlQuery query;
    query.exec(QString("SELECT MAX(last) FROM faktura WHERE vydana = %1 AND rok = %2 AND firmaid = %3")
               .arg(m_vydana?1:0)
               .arg(preferences.m_rok)
               .arg(preferences.m_firmaid));
    if(query.first())
    {
        return query.value(0).toInt()+1;
    }
    return 1;
}

bool FakturaDialog::isPlatce()
{
    QSqlQuery q;
    if(m_edit)
    {
        q.exec(QString("SELECT platce FROM faktura WHERE id=%1").arg(m_id));
        q.first();
        return q.value(0).toBool();
    }
    else
    {
        q.exec(QString("SELECT platce FROM firma WHERE id=%1").arg(preferences.m_firmaid));
        q.first();
        return q.value(0).toBool();
    }
}

int FakturaDialog::getUhrada()
{
    QSqlQuery q;
    q.prepare("SELECT uhradaid FROM faktura WHERE id=?");
    q.addBindValue(m_id);
    q.exec();
    q.first();
    return q.value(0).toInt();
}

bool FakturaDialog::isUhradaPokladnou()
{
    QSqlQuery q;
    q.exec(QString("SELECT hotove FROM fauhrada WHERE id = %1").arg(m_uhrada->itemData(m_uhrada->currentIndex()).toInt()));
    q.first();
    return q.value(0).toBool();
}

int FakturaDialog::getBanka()
{
    QSqlQuery q;
    q.prepare("SELECT bankaid FROM faktura WHERE id=?");
    q.addBindValue(m_id);
    q.exec();
    q.first();
    return q.value(0).toInt();
}

int FakturaDialog::getMena()
{
    QSqlQuery q;
    q.prepare("SELECT menaid FROM faktura WHERE id=?");
    q.addBindValue(m_id);
    q.exec();
    q.first();
    return q.value(0).toInt();
}

QDate FakturaDialog::getDate(const QString &column)
{
    if(!m_edit)
    {
        QDate current = QDate::currentDate();
        if(QString::compare(column,"splatnost"))
            return current;
        return current.addDays(preferences.m_splatnost);
    }
    QSqlQuery q;
    q.exec(QString("SELECT %1 FROM faktura WHERE id=%2").arg(column).arg(m_id));
    q.first();
    return QDate::fromString(q.value(0).toString(), "dd.MM.yyyy");
}

void FakturaDialog::slotAddUhrada()
{
    FauhradaDialog *dialog = new FauhradaDialog(this);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted && !dialog->popis().isEmpty())
    {
        int id = dxutils::getLastID("fauhrada");
        QSqlQuery q;
        q.prepare("INSERT INTO fauhrada (id,uhrada,hotove) VALUES (?,?,?)");
        q.addBindValue(id);
        q.addBindValue(dialog->popis());
        q.addBindValue(dialog->hotove());
        q.exec();
        m_uhrada->addItem(dialog->popis(), id);
        m_uhrada->setCurrentIndex(m_uhrada->count()-1);
    }
}

void FakturaDialog::slotAddMena()
{
    bool ok;
    QString mena = QInputDialog::getText(this, QString::fromUtf8("Měna"), QString::fromUtf8("Nová měna"), QLineEdit::Normal, "", &ok);
    if(ok && !mena.isEmpty())
    {
        int id = dxutils::getLastID("mena");
        QSqlQuery q;
        q.prepare("INSERT INTO mena (id,zkratka) VALUES (?,?)");
        q.addBindValue(id);
        q.addBindValue(mena);
        q.exec();
        m_mena->addItem(mena, id);
        m_mena->setCurrentIndex(m_mena->count()-1);
    }
}

void FakturaDialog::slotAddBanka()
{
    BankaDialog *dialog = new BankaDialog(this);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        int id = dxutils::getLastID("banka");
        QSqlQuery q;
        q.prepare("INSERT INTO banka (id,cislo,banka) VALUES (?,?,?)");
        q.addBindValue(id);
        q.addBindValue(dialog->cislo());
        q.addBindValue(dialog->banka());
        q.exec();
        m_banka->addItem(QString("%1/%2").arg(dialog->cislo()).arg(dialog->banka()), id);
        m_banka->setCurrentIndex(m_banka->count()-1);
    }
}

void FakturaDialog::slotAddPolozka()
{
    PolozkaDialog *dialog = new PolozkaDialog(isPlatce(), m_id, this);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        m_modelPolozka->setQuery(QString("SELECT id,popis,pocet,mj,cenamj,celkem FROM polozka WHERE fakturaid = %1").arg(m_id));
        updateCelkem();
        if(isPlatce()) updateDPH();
        m_editPolozka->setEnabled(true);
        m_removePolozka->setEnabled(true);
    }
}

void FakturaDialog::slotEditPolozka()
{
    QModelIndex current = m_polozky->currentIndex();
    int id;
    if(current.isValid())
        id = m_modelPolozka->record(current.row()).value("id").toInt();
    else
        id = m_modelPolozka->record(0).value("id").toInt();
    PolozkaDialog *dialog = new PolozkaDialog(id, isPlatce(), this);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        m_modelPolozka->setQuery(QString("SELECT id,popis,pocet,mj,cenamj,celkem FROM polozka WHERE fakturaid = %1").arg(m_id));
        updateCelkem();
        if(isPlatce()) updateDPH();
    }
}

void FakturaDialog::slotRemovePolozka()
{
    QModelIndex current = m_polozky->currentIndex();
    int id;
    if(current.isValid())
        id = m_modelPolozka->record(current.row()).value("id").toInt();
    else
        id = m_modelPolozka->record(0).value("id").toInt();
    if(QMessageBox::question(this, "Dotaz", QString("Opravdu smazat '%1'?")
        .arg(m_modelPolozka->record(current.isValid()?current.row():0).value("popis").toString()),
        QMessageBox::Ok|QMessageBox::Cancel) == QMessageBox::Ok)
    {
        QSqlQuery p(QString("DELETE FROM polozka WHERE id=%1").arg(id));
        p.exec();
        m_modelPolozka->setQuery(QString("SELECT id,popis,pocet,mj,cenamj,celkem FROM polozka WHERE fakturaid = %1").arg(m_id));
        updateCelkem();
        if(isPlatce()) updateDPH();
        if(!m_modelPolozka->rowCount())
        {
            m_editPolozka->setEnabled(false);
            m_removePolozka->setEnabled(false);
        }
    }
}

void FakturaDialog::slotDoubleclickPolozky(const QModelIndex &index)
{
    PolozkaDialog *dialog = new PolozkaDialog(m_modelPolozka->record(index.row()).value("id").toInt(), isPlatce(), this);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        updateCelkem();
        if(isPlatce()) updateDPH();
        m_modelPolozka->setQuery(QString("SELECT id,popis,pocet,mj,cenamj,celkem FROM polozka WHERE fakturaid = %1").arg(m_id));
    }
}

void FakturaDialog::slotVystaveniChanged(const QDate &date)
{
    m_duzp->setDate(date);
    m_splatnost->setDate(date.addDays(preferences.m_splatnost));
}

void FakturaDialog::updateCelkem()
{
    QSqlQuery q;
    q.exec(QString("SELECT SUM(celkem) FROM polozka WHERE fakturaid = %1").arg(m_id));
    q.first();
    m_celkem->setText(QString::fromUtf8("<b>%L1</b>").arg(q.value(0).toFloat(), 0,'f',2));
}

void FakturaDialog::updateDPH()
{
    if(m_modelPolozka->rowCount())
    {
        int frameStyle = QFrame::Sunken|QFrame::Panel;
        while(m_dphLabels.size())
        {
            m_dphLabels.at(0)->setParent(0);
            m_dphLabels.at(0)->deleteLater();
            m_dphLabels.pop_front();
        }
        int i = 1;
        QSqlQuery q;
        q.exec(QString("SELECT sum(celkem-dph) AS zaklad, sum(dph) AS dph, sum(celkem) AS celkem, sazba "
                        "FROM polozka INNER JOIN dphsazby ON sazbaid=dphsazby.id WHERE fakturaid=%1 GROUP BY sazbaid ORDER BY sazba DESC").arg(m_id));
        while(q.next())
        {            
            QLabel *sazba = new QLabel;
            sazba->setText(QString("%1%").arg(q.value(3).toInt()));
            sazba->setFrameStyle(frameStyle);
            m_dphLabels.append(sazba);
            m_dphGrid->addWidget(sazba, i, 0);
            QLabel *zaklad = new QLabel;
            zaklad->setText(QString("%L1").arg(q.value(0).toFloat(),0,'f',2));
            zaklad->setFrameStyle(frameStyle);
            m_dphLabels.append(zaklad);
            m_dphGrid->addWidget(zaklad, i, 1);
            QLabel *dph = new QLabel;
            dph->setText(QString("%L1").arg(q.value(1).toFloat(),0,'f',2));
            dph->setFrameStyle(frameStyle);
            m_dphLabels.append(dph);
            m_dphGrid->addWidget(dph, i, 2);
            QLabel *celkem = new QLabel;
            celkem->setText(QString("%L1").arg(q.value(2).toFloat(),0,'f',2));
            celkem->setFrameStyle(frameStyle);
            m_dphLabels.append(celkem);
            m_dphGrid->addWidget(celkem, i, 3);
            i++;
        }
        m_dph->show();
    }
    else
    {
        m_dph->hide();
    }
}

void FakturaDialog::slotPopup(const QPoint &pos)
{
    QModelIndex current = m_polozky->currentIndex();
    if(current.isValid())
    {
        QMenu popup;
        popup.addAction(m_editPolozka);
        popup.addAction(m_removePolozka);
        popup.exec(m_polozky->mapToGlobal(pos));
    }
}

void FakturaDialog::slotUhrada(int index)
{
    QSqlQuery q;
    q.exec(QString("SELECT hotove FROM fauhrada WHERE id = %1").arg(m_uhrada->itemData(index).toInt()));
    q.first();
    if(q.value(0).toBool())
    {
        m_labelBanka->hide();
        m_banka->hide();
        m_bankaAdd->hide();
    }
    else
    {
        m_labelBanka->show();
        m_banka->show();
        m_bankaAdd->show();
    }
}

