/***************************************************************
 * Name:      findreplacedialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "findreplacedialog.h"

FindreplaceDialog::FindreplaceDialog(QWidget *parent, QTextEdit *edit, bool replace) :
    QDialog(parent), m_showReplace(replace), m_edit(edit)
{
    QGridLayout *grid = new QGridLayout;
    QLabel *label = new QLabel(QString::fromUtf8("Na&jít"));
    m_find = new QLineEdit;
    label->setBuddy(m_find);
    grid->addWidget(label, 0, 0);
    grid->addWidget(m_find, 0, 1);
    label = new QLabel("Na&hradit");
    m_replace = new QLineEdit;
    label->setBuddy(m_replace);
    if(m_showReplace)
    {
        grid->addWidget(label, 1, 0);
        grid->addWidget(m_replace, 1, 1);
    }

    m_caseInsensitive = new QCheckBox(QString::fromUtf8("Ignorovat velikost písmen"));
    m_caseInsensitive->setChecked(true);
    m_wholeWords = new QCheckBox(QString::fromUtf8("Celá slova"));
    m_regularExpresion = new QCheckBox(QString::fromUtf8("Regulární výrazy"));
    QHBoxLayout *checkLayout = new QHBoxLayout;
    checkLayout->addWidget(m_caseInsensitive);
    checkLayout->addWidget(m_wholeWords);
    checkLayout->addWidget(m_regularExpresion);

    m_warning = new QLabel;
    m_warning->setStyleSheet("color: darkred; font-weight: bold");

    QVBoxLayout *leftLayout = new QVBoxLayout;
    leftLayout->addLayout(grid);
    leftLayout->addLayout(checkLayout);
    leftLayout->addWidget(m_warning);

    m_findNext = new QPushButton(QString::fromUtf8("&Najít další"));
    m_findNext->setDefault(true);
    connect(m_findNext, SIGNAL(clicked()), this, SLOT(slotFindNext()));
    m_findPrevious = new QPushButton(QString::fromUtf8("Najít &předchozí"));
    connect(m_findPrevious, SIGNAL(clicked()), this, SLOT(slotFindPrevious()));
    m_findFromTop = new QPushButton(QString::fromUtf8("Hledat od &začátku"));
    connect(m_findFromTop, SIGNAL(clicked()), this, SLOT(slotFindFromTop()));
    m_replaceFindNext = new QPushButton(QString::fromUtf8("Nahradit a\nnajít &další"));
    connect(m_replaceFindNext, SIGNAL(clicked()), this, SLOT(slotReplaceFindNext()));
    m_replaceFindPrevious = new QPushButton(QString::fromUtf8("Nahradit a\nnajít před&chozí"));
    connect(m_replaceFindPrevious, SIGNAL(clicked()), this, SLOT(slotReplaceFindPrevious()));
    m_replaceAll = new QPushButton(QString::fromUtf8("Nahradit &vše"));
    connect(m_replaceAll, SIGNAL(clicked()), this, SLOT(slotReplaceAll()));
    QVBoxLayout *buttonLayout = new QVBoxLayout;
    buttonLayout->addWidget(m_findNext);
    buttonLayout->addWidget(m_findPrevious);
    buttonLayout->addWidget(m_findFromTop);
    if(m_showReplace)
    {
        buttonLayout->addWidget(m_replaceFindNext);
        buttonLayout->addWidget(m_replaceFindPrevious);
        buttonLayout->addWidget(m_replaceAll);
    }

    QHBoxLayout *mainLayout = new QHBoxLayout;
    mainLayout->addLayout(leftLayout);
    mainLayout->addLayout(buttonLayout);
    setLayout(mainLayout);
    setWindowTitle(m_showReplace ? QString::fromUtf8("Najít a nahradit") : QString::fromUtf8("Najít"));
}

bool FindreplaceDialog::find(const QTextCursor pos, const QTextDocument::FindFlag flag)
{
    Q_ASSERT(m_edit);
    QTextDocument::FindFlags flags(flag);
    QTextCursor nc;
    if(m_wholeWords->isChecked())
        flags |= QTextDocument::FindWholeWords;
    if(m_regularExpresion->isChecked())
    {
        Qt::CaseSensitivity cs;
        if(m_caseInsensitive->isChecked())
            cs = Qt::CaseInsensitive;
        else
            cs = Qt::CaseSensitive;
        nc = m_edit->document()->find(QRegExp(m_find->text(), cs), pos, flags);
    }
    else
    {
        if(!m_caseInsensitive->isChecked())
            flags |= QTextDocument::FindCaseSensitively;
        nc = m_edit->document()->find(m_find->text(), pos,flags);
    }
    if(nc.isNull())
    {
        nc = QTextCursor(m_edit->document());
        if(flag == QTextDocument::FindBackward)
            nc.movePosition(QTextCursor::End);
        m_warning->setText(QString::fromUtf8("Dosažen konec dokumentu!"));
        m_edit->setTextCursor(nc);
        return false;
    }
    else
    {
        m_warning->setText("");
        m_edit->setTextCursor(nc);
        return true;
    }
}

void FindreplaceDialog::replace()
{
    QTextCursor cc(m_edit->textCursor());
    if(cc.isNull() || !cc.hasSelection())
        return;
    if(m_showReplace)
    {
        const int p = qMin(cc.anchor(), cc.position());
        const int l = m_replace->text().length();
        cc.insertText(m_replace->text());
        cc.setPosition(p);
        cc.setPosition(p+l, QTextCursor::KeepAnchor);
        m_edit->setTextCursor(cc);
    }
}

void FindreplaceDialog::slotFindNext()
{
    find(m_edit->textCursor(), QTextDocument::FindFlag(0));
}

void FindreplaceDialog::slotFindPrevious()
{
    find(m_edit->textCursor(), QTextDocument::FindBackward);
}

void FindreplaceDialog::slotFindFromTop()
{
    find(QTextCursor(m_edit->document()), QTextDocument::FindFlag(0));
}

void FindreplaceDialog::slotReplaceFindNext()
{
    replace();
    find(m_edit->textCursor(), QTextDocument::FindFlag(0));
}

void FindreplaceDialog::slotReplaceFindPrevious()
{
    replace();
    find(m_edit->textCursor(), QTextDocument::FindBackward);
}

void FindreplaceDialog::slotReplaceAll()
{
    QTextCursor c(m_edit->document());
    while(find(c, QTextDocument::FindFlag(0)))
    {
        replace();
        c = m_edit->textCursor();
    }
}
