/***************************************************************
 * Name:      firmadialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include "firmadialog.h"
#include "dxutils.h"

FirmaDialog::FirmaDialog(bool edit, int id, QWidget *parent) :
    QDialog(parent), m_id(id), m_edit(edit)
{
    QDialogButtonBox *buttons;
    buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    QGridLayout *grid = new QGridLayout;
    grid->addWidget(new QLabel(QString::fromUtf8("Název")), 0, 0);
    m_nazev = new QLineEdit;
    grid->addWidget(m_nazev, 0, 1, 1, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("Ulice")), 1, 0);
    m_ulice = new QLineEdit;
    grid->addWidget(m_ulice, 1, 1);
    grid->addWidget(new QLabel(QString::fromUtf8("čp")), 1, 2);
    m_cp = new QLineEdit;
    grid->addWidget(m_cp, 1, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("PSČ")), 2, 0);
    m_psc = new QLineEdit;
    grid->addWidget(m_psc, 2, 1);
    grid->addWidget(new QLabel(QString::fromUtf8("Místo")), 2, 2);
    m_misto = new QLineEdit;
    grid->addWidget(m_misto, 2, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("Země")), 3, 0);
    m_zeme = new QLineEdit;
    grid->addWidget(m_zeme, 3, 1, 1, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("IČ")), 4, 0);
    m_ic = new QLineEdit;
    grid->addWidget(m_ic, 4, 1, 1, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("DIČ")), 5, 0);
    m_dic = new QLineEdit;
    grid->addWidget(m_dic, 5, 1, 1, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("Registrace")), 6, 0);
    m_registrace = new QLineEdit;
    grid->addWidget(m_registrace, 6, 1, 1, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("Plátce DPH")), 7, 0);
    m_platce = new QCheckBox;
    grid->addWidget(m_platce, 7, 1);
    grid->addWidget(new QLabel(QString::fromUtf8("Telefon")), 8, 0);
    m_telefon = new QLineEdit;
    grid->addWidget(m_telefon, 8, 1, 1, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("E-mail")), 9, 0);
    m_email = new QLineEdit;
    grid->addWidget(m_email, 9, 1, 1, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("WWW")), 10, 0);
    m_www = new QLineEdit;
    grid->addWidget(m_www, 10, 1, 1, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("Logo")), 11, 0);
    m_logoPath = new QLineEdit;
    grid->addWidget(m_logoPath, 11, 1, 1, 2);
    m_logoSelect = new QPushButton("...");
    connect(m_logoSelect, SIGNAL(clicked()), this, SLOT(onLogoSelect()));
    grid->addWidget(m_logoSelect, 11, 3);
    grid->addWidget(new QLabel(QString::fromUtf8("Razítko")), 12, 0);
    m_razitkoPath = new QLineEdit;
    grid->addWidget(m_razitkoPath, 12, 1, 1, 2);
    m_razitkoSelect = new QPushButton("...");
    connect(m_razitkoSelect, SIGNAL(clicked()), this, SLOT(onRazitkoSelect()));
    grid->addWidget(m_razitkoSelect, 12, 3);
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addLayout(grid);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    if(!m_edit && m_id == 1 ) setWindowTitle("Zadejte firmu"); //first run
    else setWindowTitle("Firma");
    setupGUI();
}

void FirmaDialog::setupGUI()
{
    if(m_edit)
    {
        QSqlQuery q;
        q.exec(QString("SELECT nazev,ic,dic,registrace,platce,telefon,email,www,logo,razitko,ulice,cp,misto,psc,zeme FROM firma_view WHERE id=%1").arg(m_id));
        q.first();
        m_nazev->setText(q.value(0).toString());
        m_ic->setText(q.value(1).toString());
        m_dic->setText(q.value(2).toString());
        m_registrace->setText(q.value(3).toString());
        m_platce->setChecked(q.value(4).toBool());
        m_telefon->setText(q.value(5).toString());
        m_email->setText(q.value(6).toString());
        m_www->setText(q.value(7).toString());
        m_logoPath->setText(q.value(8).toString());
        m_razitkoPath->setText(q.value(9).toString());
        m_ulice->setText(q.value(10).toString());
        m_cp->setText(q.value(11).toString());
        m_misto->setText(q.value(12).toString());
        m_psc->setText(q.value(13).toString());
        m_zeme->setText(q.value(14).toString());
    }
    else
    {
        m_zeme->setText(QString::fromUtf8("Česká republika"));
    }
}

bool FirmaDialog::checkNazev(const QString &nazev)
{
    return !nazev.isEmpty();
}

void FirmaDialog::accept()
{
    if(!dxutils::checkIC(m_ic->text())) QMessageBox::information(this, "Informace", QString::fromUtf8("IČ není platné"), QMessageBox::Ok);
    if(checkNazev(m_nazev->text()))
    {
        if(m_edit)
        {
            QSqlQuery q;
            q.prepare("UPDATE firma_view SET nazev=?,ic=?,dic=?,registrace=?,platce=?,telefon=?,"
                      "email=?,www=?,logo=?,razitko=?,ulice=?,cp=?,misto=?,psc=?,zeme=? WHERE id=?");
            q.addBindValue(m_nazev->text());
            q.addBindValue(m_ic->text());
            q.addBindValue(m_dic->text());
            q.addBindValue(m_registrace->text());
            q.addBindValue(m_platce->isChecked());
            q.addBindValue(m_telefon->text());
            q.addBindValue(m_email->text());
            q.addBindValue(m_www->text());
            q.addBindValue(m_logoPath->text());
            q.addBindValue(m_razitkoPath->text());
            q.addBindValue(m_ulice->text());
            q.addBindValue(m_cp->text());
            q.addBindValue(m_misto->text());
            q.addBindValue(m_psc->text());
            q.addBindValue(m_zeme->text());
            q.addBindValue(m_id);
            if(!q.exec())
                QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
        }
        else
        {
            int adresaid = dxutils::getLastID("adresa");
            QSqlQuery q;
            q.prepare("INSERT INTO adresa (id,ulice,cp,misto,psc,zeme) VALUES (?,?,?,?,?,?)");\
            q.addBindValue(adresaid);
            q.addBindValue(m_ulice->text());
            q.addBindValue(m_cp->text());
            q.addBindValue(m_misto->text());
            q.addBindValue(m_psc->text());
            q.addBindValue(m_zeme->text());
            if(!q.exec())
                QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
            int firmaid = dxutils::getLastID("firma");
            q.prepare("INSERT INTO firma (id,nazev,ic,dic,registrace,platce,telefon,email,www,logo,razitko,adresaid)"
                      " VALUES (?,?,?,?,?,?,?,?,?,?,?,?)");
            q.addBindValue(firmaid);
            q.addBindValue(m_nazev->text());
            q.addBindValue(m_ic->text());
            q.addBindValue(m_dic->text());
            q.addBindValue(m_registrace->text());
            q.addBindValue(m_platce->isChecked());
            q.addBindValue(m_telefon->text());
            q.addBindValue(m_email->text());
            q.addBindValue(m_www->text());
            q.addBindValue(m_logoPath->text());
            q.addBindValue(m_razitkoPath->text());
            q.addBindValue(adresaid);
            if(!q.exec())
                QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
            q.prepare("INSERT INTO settings (id,rok,vydana,prijata,splatnost,firmaid,"
                      "faprijataneplatce,faprijataplatce,favydananeplatce,favydanaplatce,zalohaprijataneplatce,"
                      "zalohaprijataplatce,zalohavydananeplatce,zalohavydanaplatce)"
                      " VALUES (?,?,?,?,?,?,"
                      "'internal','internal','internal','internal','internal',"
                      "'internal','internal','internal')");
            q.addBindValue(dxutils::getLastID("settings"));
            q.addBindValue(QDate::currentDate().toString("yyyy").toInt());
            q.addBindValue(QDate::currentDate().toString("yyyy"));
            q.addBindValue(QDate::currentDate().toString("yyyy"));
            q.addBindValue(14);
            q.addBindValue(firmaid);
            if(!q.exec())
                QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
            q.prepare(QString::fromUtf8("INSERT INTO smtp (id,server,port,ssl,autentifikace,username,password,"
                      "sender,subject,text,firmaid)"
                      " VALUES(?,'',25,0,0,'','','Jan Novak <novak@novakov.cz>','Faktura','Dobrý den,\nv příloze Vám zasíláme fakturu',?)"));
            q.addBindValue(dxutils::getLastID("smtp"));
            q.addBindValue(firmaid);
            if(!q.exec())
                QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
        }
        QDialog::accept();
    }
    else
    {
        if(!m_edit && m_id == 1 ) QMessageBox::warning(this, "Informace", QString::fromUtf8("Název nemůže být prázdný"), QMessageBox::Ok); //first run
        else QMessageBox::information(this, "Informace", QString::fromUtf8("Název nemůže být prázdný"), QMessageBox::Ok);
    }
}

void FirmaDialog::reject()
{
    if(!m_edit && m_id == 1 ) //first run
    {
        QMessageBox::warning(this, "Informace", QString::fromUtf8("Název nemůže být prázdný"), QMessageBox::Ok);
    }
    else
    {
        QDialog::reject();
    }
}

void FirmaDialog::keyPressEvent(QKeyEvent *ev)
{
    if(!m_edit)
    {
        if(ev->key() != Qt::Key_Escape)
            QDialog::keyPressEvent(ev);
    }
    else
    {
        QDialog::keyPressEvent(ev);
    }
}

void FirmaDialog::onLogoSelect()
{
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
    QString file = QFileDialog::getOpenFileName(this,
                                "Vyberte logo",
                                m_logoPath->text(),
                                "",
                                NULL,
                                options);
    if(!file.isEmpty())
    {
        m_logoPath->setText(file);
    }
}

void FirmaDialog::onRazitkoSelect()
{
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
    QString file = QFileDialog::getOpenFileName(this,
                                QString::fromUtf8("Vyberte razítko"),
                                m_razitkoPath->text(),
                                "",
                                NULL,
                                options);
    if(!file.isEmpty())
    {
         m_razitkoPath->setText(file);
    }
}
