/***************************************************************
 * Name:      firmydialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include "firmydialog.h"
#include "firmadialog.h"
#include "dxutils.h"

FirmyDialog::FirmyDialog(QWidget *parent) :
    QDialog(parent), m_selectedID(0)
{
    setupModel();
    QAction *add = new QAction(QIcon(":/icons/add.png"), QString::fromUtf8("Přidat"), this);
    add->setToolTip(QString::fromUtf8("Přidat firmu"));
    connect(add, SIGNAL(triggered()), this, SLOT(slotAdd()));
    m_edit = new QAction(QIcon(":/icons/edit.png"), QString::fromUtf8("Upravit"), this);
    m_edit->setToolTip(QString::fromUtf8("Upravit firmu"));
    connect(m_edit, SIGNAL(triggered()), this, SLOT(slotEdit()));
    m_remove = new QAction(QIcon(":/icons/delete.png"), QString::fromUtf8("Smazat"), this);
    m_remove->setToolTip(QString::fromUtf8("Smazat firmu"));
    connect(m_remove, SIGNAL(triggered()), this, SLOT(slotRemove()));
    if(m_model->rowCount()<2) m_remove->setEnabled(false);
    if(!m_model->rowCount()) m_edit->setEnabled(false);
    QToolBar *bar = new QToolBar("Firmy", this);
    bar->setStyleSheet("QToolBar { border: 0px }");
    bar->setMovable(false);
    bar->addAction(add);
    bar->addAction(m_edit);
    bar->addAction(m_remove);
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    m_firmy = new QListView;
    m_firmy->setModel(m_model);
    m_firmy->setModelColumn(1);
    m_firmy->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(m_firmy, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(slotDoubleclickFirmy(QModelIndex)));
    connect(m_firmy, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(slotPopup(QPoint)));
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(bar);
    mainLayout->addWidget(m_firmy);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle("Firmy");
}

void FirmyDialog::setupModel()
{
    m_model = new QSqlQueryModel(this);
    m_model->setQuery("SELECT * FROM firma");
}

void FirmyDialog::slotAdd()
{
    FirmaDialog *dialog = new FirmaDialog(false, 0, this);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        m_model->setQuery("SELECT * FROM firma");
        if(m_model->rowCount())
        {
            m_edit->setEnabled(true);
            if(m_model->rowCount()>1) m_remove->setEnabled(true);
        }
    }
}

void FirmyDialog::slotEdit()
{
    QModelIndex current = m_firmy->currentIndex();
    int id;
    if(current.isValid())
        id = m_model->record(current.row()).value("id").toInt();
    else
        id = m_model->record(0).value("id").toInt();
    FirmaDialog *dialog = new FirmaDialog(true, id, this);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        m_model->setQuery("SELECT * FROM firma");
    }
}

void FirmyDialog::slotDoubleclickFirmy(const QModelIndex &index)
{
    FirmaDialog *dialog = new FirmaDialog(true, m_model->record(index.row()).value("id").toInt(), this);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        m_model->setQuery("SELECT * FROM firma");
    }
}

void FirmyDialog::slotRemove()
{
    QModelIndex current = m_firmy->currentIndex();
    int id;
    QString name;
    if(current.isValid())
    {
        id = m_model->record(current.row()).value("id").toInt();
        name = m_model->record(current.row()).value("nazev").toString();
    }
    else
    {
        id = m_model->record(0).value("id").toInt();
        name = m_model->record(0).value("nazev").toString();
    }
    if(QMessageBox::question(this, "Dotaz", QString("Opravdu smazat firmu '%1'?").arg(name), QMessageBox::Ok|QMessageBox::Cancel) == QMessageBox::Ok)
    {
        QSqlQuery q;
        q.prepare("DELETE FROM settings WHERE firmaid = ?");
        q.addBindValue(id);
        q.exec();
        q.prepare("DELETE FROM firma WHERE id = ?");
        q.addBindValue(id);
        q.exec();
        m_model->setQuery("SELECT * FROM firma");
        if(m_model->rowCount()<2) m_remove->setEnabled(false);
        if(!m_model->rowCount()) m_edit->setEnabled(false);
    }
}

int FirmyDialog::selectedID()
{
    QModelIndex current = m_firmy->currentIndex();
    if(current.isValid())
        m_selectedID = m_model->record(current.row()).value("id").toInt();
    else
        m_selectedID = m_model->record(0).value("id").toInt();
    return m_selectedID;
}

void FirmyDialog::slotPopup(const QPoint &pos)
{
    QModelIndex current = m_firmy->currentIndex();
    if(current.isValid())
    {
        QMenu popup;
        popup.addAction(m_edit);
        if(m_model->rowCount()>1)
        {
            popup.addAction(m_remove);
        }
        popup.exec(m_firmy->mapToGlobal(pos));
    }
}

