/***************************************************************
 * Name:      helpdialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include "widgets/helpbrowser.h"
#include "helpdialog.h"
#include "dxutils.h"

const char* INDEX_URL = "qthelp://org.dxsolutions.dxfaktura/doc/files/home.html";

HelpDialog::HelpDialog(QWidget *parent) :
    QDialog(parent),
    m_tmpDir(QString("dxfaktura-%1").arg(QCoreApplication::applicationPid())),
    m_qchFile(QDir::temp().filePath(m_tmpDir + "/dxfaktura.qch")),
    m_qhcFile(QDir::temp().filePath(m_tmpDir + "/dxfaktura.qhc"))
{
    QDir::temp().mkdir(m_tmpDir);
    QFile(":/doc/dxfaktura.qhc").copy(m_qhcFile);
    QFile(":/doc/dxfaktura.qch").copy(m_qchFile);
    m_engine = new QHelpEngine(m_qhcFile, this);
    if(!m_engine->setupData())
    {
        QMessageBox::critical(this, QString::fromUtf8("Nápověda"), QString::fromUtf8("Nemohu otevřít dokumentaci"));
        deleteLater();
        return;
    }
    QSplitter *splitter = new QSplitter;
    QHelpContentWidget* content = m_engine->contentWidget();
    splitter->insertWidget(0, content);
    splitter->setSizes(QList<int>() << 80 << 400);
    content->setRootIsDecorated(false);
    connect(content, SIGNAL(linkActivated(const QUrl&)), this, SLOT(openPage(const QUrl&)));
    connect(content, SIGNAL(clicked(const QModelIndex&)), this, SLOT(itemClicked(const QModelIndex&)));
    m_text = new HelpBrowser(this);
    m_text->setEngine(m_engine);
    splitter->insertWidget(1, m_text);
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(splitter);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    resize(600, 400);
    setWindowTitle(QString::fromUtf8("Nápověda"));
    openPage(QUrl(INDEX_URL));
    QTimer::singleShot(100, content, SLOT(expandAll()));
}

HelpDialog::~HelpDialog()
{
    QFile(m_qchFile).remove();
    QFile(m_qhcFile).remove();
    QDir::temp().rmdir(m_tmpDir);
}

void HelpDialog::openPage(const QUrl &url)
{
    m_text->setSource(url);
}

void HelpDialog::itemClicked(const QModelIndex &index)
{
    openPage(m_engine->contentModel()->contentItemAt(index)->url());
}
