/***************************************************************
 * Name:      optionsdialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <QtSql>
#include "optionsdialog.h"
#include "preferences.h"
#include "dxutils.h"

OptionsDialog::OptionsDialog(QWidget *parent) :
    QDialog(parent)
{
    QVBoxLayout *mainLayout = new QVBoxLayout;
    QSqlQuery q(QString("SELECT nazev FROM firma WHERE id = %1").arg(preferences.m_firmaid));
    q.first();
    QLabel *firma = new QLabel;
    firma->setText(QString("Firma: <b>%1</b></big>").arg(q.value(0).toString()));
    firma->setAlignment(Qt::AlignCenter);
    mainLayout->addWidget(firma);
    QTabWidget *tabbook = new QTabWidget(this);
    createFakturaceTab(tabbook);
    createSmtpTab(tabbook);
    mainLayout->addWidget(tabbook);
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle(QString::fromUtf8("Nastavení"));
    setupGUI();
}

void OptionsDialog::createFakturaceTab(QTabWidget *tabbook)
{
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    QGridLayout *grid = new QGridLayout;
    grid->addWidget(new QLabel(QString::fromUtf8("Účetní rok")),0,0);
    m_rok = new QSpinBox;
    m_rok->setRange(1989,QDate::currentDate().year()+1);
    connect(m_rok, SIGNAL(editingFinished()), this, SLOT(slotRok()));
    grid->addWidget(m_rok,0,1);
    grid->addWidget(new QLabel(QString::fromUtf8("Prefix vydaných faktur")),1,0);
    m_vydana = new QLineEdit;
    grid->addWidget(m_vydana,1,1);
    grid->addWidget(new QLabel(QString::fromUtf8("Prefix přijatých faktur")),2,0);
    m_prijata = new QLineEdit;
    grid->addWidget(m_prijata,2,1);
    grid->addWidget(new QLabel("Splatnost faktur"),3,0);
    m_splatnost = new QSpinBox;
    grid->addWidget(m_splatnost,3,1);
    main->addLayout(grid);
    tab->setLayout(main);
    tabbook->addTab(tab, "&Fakturace");
}

void OptionsDialog::createSmtpTab(QTabWidget *tabbook)
{
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    QGridLayout *grid = new QGridLayout;
    grid->addWidget(new QLabel("SMTP server"),0,0);
    m_server = new QLineEdit;
    grid->addWidget(m_server,0,1);
    grid->addWidget(new QLabel("Port"),1,0);
    m_port = new QSpinBox;
    m_port->setRange(0,65535);
    grid->addWidget(m_port,1,1);
    grid->addWidget(new QLabel("SSL"),2,0);
    m_ssl = new QCheckBox;
    grid->addWidget(m_ssl,2,1);
    grid->addWidget(new QLabel(QString::fromUtf8("Použít autentifikaci")),3,0);
    m_autentifikace = new QCheckBox;
    grid->addWidget(m_autentifikace,3,1);
    grid->addWidget(new QLabel(QString::fromUtf8("Uživatelské jméno")),4,0);
    m_username = new QLineEdit;
    grid->addWidget(m_username,4,1);
    grid->addWidget(new QLabel("Heslo"),5,0);
    m_password = new QLineEdit;
    m_password->setEchoMode(QLineEdit::PasswordEchoOnEdit);
    grid->addWidget(m_password,5,1);
    grid->addWidget(new QLabel(QString::fromUtf8("Odesílatel")),6,0);
    m_sender = new QLineEdit;
    grid->addWidget(m_sender,6,1);
    grid->addWidget(new QLabel(QString::fromUtf8("Předmět")),7,0);
    m_subject = new QLineEdit;
    grid->addWidget(m_subject,7,1);
    grid->addWidget(new QLabel("Text"),8,0);
    m_text = new QTextEdit;
    m_text->setMaximumHeight(50);
    grid->addWidget(m_text,8,1);
    main->addLayout(grid);
    tab->setLayout(main);
    tabbook->addTab(tab, "&E-mail");
}

void OptionsDialog::accept()
{
    preferences.m_rok = m_rok->value();
    if(addDPH())
    {
        addSazba();
        while(QMessageBox::question(this, "Dotaz", QString::fromUtf8("Přidat další sazbu?"),
                QMessageBox::Yes|QMessageBox::No) == QMessageBox::Yes)
        {
            addSazba();
        }
    }
    preferences.m_prefixVydana = m_vydana->text();
    preferences.m_prefixPrijata = m_prijata->text();
    preferences.m_splatnost = m_splatnost->value();
    QSqlQuery q;
    q.prepare("UPDATE settings SET rok=?,vydana=?,prijata=?,splatnost=? WHERE firmaid=?");
    q.addBindValue(preferences.m_rok);
    q.addBindValue(preferences.m_prefixVydana);
    q.addBindValue(preferences.m_prefixPrijata);
    q.addBindValue(preferences.m_splatnost);
    q.addBindValue(preferences.m_firmaid);
    if(!q.exec())
        QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
    preferences.m_server = m_server->text();
    preferences.m_port = m_port->value();
    preferences.m_ssl = m_ssl->isChecked();
    preferences.m_autentifikace = m_autentifikace->isChecked();
    preferences.m_username = m_username->text();
    preferences.m_password = m_password->text();
    preferences.m_sender = m_sender->text();
    preferences.m_subject = m_subject->text();
    preferences.m_text = m_text->toPlainText();
    q.prepare("UPDATE smtp SET server=?,port=?,ssl=?,autentifikace=?,username=?,password=?,"
              "sender=?,subject=?,text=? WHERE firmaid=?");
    q.addBindValue(preferences.m_server);
    q.addBindValue(preferences.m_port);
    q.addBindValue(preferences.m_ssl);
    q.addBindValue(preferences.m_autentifikace);
    q.addBindValue(preferences.m_username);
    q.addBindValue(preferences.m_password);
    q.addBindValue(preferences.m_sender);
    q.addBindValue(preferences.m_subject);
    q.addBindValue(preferences.m_text);
    q.addBindValue(preferences.m_firmaid);
    if(!q.exec())
        QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
    QDialog::accept();
}

void OptionsDialog::slotRok()
{
    m_vydana->setText(QString::number(m_rok->value()));
    m_prijata->setText(QString::number(m_rok->value()));
}

void OptionsDialog::setupGUI()
{
    QSqlQuery q(QString("SELECT rok,vydana,prijata,splatnost FROM settings WHERE firmaid=%1").arg(preferences.m_firmaid));
    q.first();
    m_rok->setValue(q.value(0).toInt());
    m_vydana->setText(q.value(1).toString());
    m_prijata->setText(q.value(2).toString());
    m_splatnost->setValue(q.value(3).toInt());
    q.exec(QString("SELECT server,port,ssl,autentifikace,username,password,sender,subject,text FROM smtp WHERE firmaid=%1").arg(preferences.m_firmaid));
    q.first();
    m_server->setText(q.value(0).toString());
    m_port->setValue(q.value(1).toInt());
    m_ssl->setChecked(q.value(2).toBool());
    m_autentifikace->setChecked(q.value(3).toBool());
    m_username->setText(q.value(4).toString());
    m_password->setText(q.value(5).toString());
    m_sender->setText(q.value(6).toString());
    m_subject->setText(q.value(7).toString());
    m_text->setPlainText(q.value(8).toString());
}

bool OptionsDialog::addDPH()
{
    QSqlQuery q(QString("SELECT platce FROM firma WHERE id=%1").arg(preferences.m_firmaid));
    q.first();
    bool platce = q.value(0).toBool();
    if(!platce) return false;
    q.exec(QString("SELECT COUNT(id) FROM dphsazby WHERE rok=%1").arg(preferences.m_rok));
    q.first();
    bool sazba = q.value(0).toInt()==0;
    if(platce && sazba)
    {
        if(QMessageBox::question(this, QString::fromUtf8("Přidat sazbu DPH"),
            QString::fromUtf8("Pro rok %1 nejsou definovány sazby DPH.\nChcete je přidat?\n(Je možné přidat i později při vytváření faktury)").arg(preferences.m_rok),
            QMessageBox::Yes|QMessageBox::No) == QMessageBox::Yes)
            return true;
    }
    return false;
}

void OptionsDialog::addSazba()
{
    bool ok;
    int sazba = QInputDialog::getInt(this, "Sazba DPH", "Zadejte novou sazbu", 1, 1, 99, 1, &ok);
    if(ok)
    {
        ok = true;
        QSqlQuery q(QString("SELECT sazba FROM dphsazby WHERE rok=%1 ORDER BY sazba DESC").arg(preferences.m_rok));
        while(q.next())
        {
            if(sazba == q.value(1).toInt())
            {
                ok = false;
                break;
            }
        }
        if(ok)
        {
            int id = dxutils::getLastID("dphsazby");
            q.prepare("INSERT INTO dphsazby (id, sazba, rok) VALUES (?,?,?)");
            q.addBindValue(id);
            q.addBindValue(sazba);
            q.addBindValue(preferences.m_rok);
            if(!q.exec())
                QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
        }
    }
}
