/***************************************************************
 * Name:      polozkadialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include "polozkadialog.h"
#include "dxutils.h"
#include "preferences.h"

PolozkaDialog::PolozkaDialog(bool platce, int faktura, QWidget *parent) :
    QDialog(parent), m_edit(false), m_faktura(faktura), m_platce(platce)
{
    createGUI();
}

PolozkaDialog::PolozkaDialog(int id, bool platce, QWidget *parent) :
    QDialog(parent), m_edit(true), m_id(id), m_platce(platce)
{
    QSqlQuery q(QString("SELECT fakturaid FROM polozka WHERE id=%1").arg(id));
    q.first();
    m_faktura = q.value(0).toInt();
    createGUI();
}

void PolozkaDialog::createGUI()
{
    int i = 0;
    int frameStyle = QFrame::Sunken|QFrame::Panel;
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    QGridLayout *grid = new QGridLayout;
    grid->addWidget(new QLabel("Popis"),i,0);
    m_popis = new QLineEdit;
    grid->addWidget(m_popis,i,1,1,3);
    i++;
    grid->addWidget(new QLabel(QString::fromUtf8("Počet")),i,0);
    m_pocet = new QDoubleSpinBox;
    m_pocet->setMaximum(1234567);
    m_pocet->setDecimals(3);
    connect(m_pocet, SIGNAL(valueChanged(double)), this, SLOT(slotUpdateDphCelkem()));
    grid->addWidget(m_pocet,i,1);
    grid->addWidget(new QLabel("mj"),i,2);
    m_mj = new QLineEdit;
    grid->addWidget(m_mj,i,3);
    i++;
    grid->addWidget(new QLabel("Cena za mj"),i,0);
    m_cenamj = new QDoubleSpinBox;
    m_cenamj->setMaximum(123456789);
    connect(m_cenamj, SIGNAL(valueChanged(double)), this, SLOT(slotUpdateDphCelkem()));
    grid->addWidget(m_cenamj,i,1,1,3);
    i++;
    m_bezdph = new QCheckBox("bez DPH");
    m_bezdph->setChecked(true);
    if(m_platce)
    {
        connect(m_bezdph, SIGNAL(stateChanged(int)), this, SLOT(slotUpdateDphCelkem()));
        grid->addWidget(m_bezdph,i,1,1,3);
        i++;
    }
    if(m_platce) grid->addWidget(new QLabel("Sazba DPH"),i,0);
    m_sazba = new QComboBox;
    if(m_platce)
    {
        connect(m_sazba, SIGNAL(currentIndexChanged(int)), this, SLOT(slotUpdateDphCelkem()));
        grid->addWidget(m_sazba,i,1,1,2);
    }
    m_sazbaAdd = new QPushButton("...");
    m_sazbaAdd->setToolTip(QString::fromUtf8("Přidat sazbu"));
    if(m_platce)
    {
        connect(m_sazbaAdd, SIGNAL(clicked()), this, SLOT(slotAddSazba()));
        grid->addWidget(m_sazbaAdd,i,3);
        i++;
    }
    if(m_platce) grid->addWidget(new QLabel("DPH"),i,0);
    m_labelDph = new QLabel;
    m_labelDph->setFrameStyle(frameStyle);
    if(m_platce)
    {
        grid->addWidget(m_labelDph,i,1,1,3);
        i++;
    }
    grid->addWidget(new QLabel("Celkem"),i,0);
    m_labelCelkem = new QLabel;
    m_labelCelkem->setFrameStyle(frameStyle);
    grid->addWidget(m_labelCelkem,i,1,1,3);
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addLayout(grid);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle(QString::fromUtf8("Položka faktury"));
    setupGUI();
}

void PolozkaDialog::setupGUI()
{
    QSqlQuery q(QString("SELECT id,sazba FROM dphsazby WHERE rok=%1 ORDER BY sazba DESC").arg(rokFaktury()));
    while(q.next())
    {
        m_sazba->addItem(QString("%1%").arg(q.value(1).toInt()), q.value(0).toInt());
    }
    m_sazba->addItem("0%", 0);
    if(m_edit)
    {
        q.exec(QString("SELECT popis,mj,pocet,cenamj,sazbaid,bezdph FROM polozka WHERE id=%1").arg(m_id));
        q.first();
        m_popis->setText(q.value(0).toString());
        m_mj->setText(q.value(1).toString());
        m_pocet->setValue(q.value(2).toDouble());
        m_cenamj->setValue(q.value(3).toDouble());
        int sazbaid = q.value(4).toInt();
        for(int i=0; i<m_sazba->count(); i++)
        {
            if(m_sazba->itemData(i) == sazbaid)
            {
                m_sazba->setCurrentIndex(i);
                break;
            }
            m_sazba->setCurrentIndex(m_sazba->count()-1);
        }
        if(m_platce)
        {
            if(q.value(5).toBool()) m_bezdph->setChecked(true);
            else m_bezdph->setChecked(false);
        }
        slotUpdateDphCelkem();
    }
}

void PolozkaDialog::accept()
{
    if(m_edit)
    {
        QSqlQuery q;
        q.prepare("UPDATE polozka SET popis=?,mj=?,pocet=?,cenamj=?,sazbaid=?,dph=?,"
                  "bezdph=?,celkem=? WHERE id=?");
        q.addBindValue(m_popis->text());
        q.addBindValue(m_mj->text());
        q.addBindValue(m_pocet->value());
        q.addBindValue(m_cenamj->value());
        q.addBindValue(m_sazba->itemData(m_sazba->currentIndex()));
        q.addBindValue(m_dph);
        q.addBindValue(m_bezdph->isChecked());
        q.addBindValue(m_celkem);
        q.addBindValue(m_id);
        if(!q.exec())
            QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
    }
    else
    {
        QSqlQuery q;
        q.prepare("INSERT INTO polozka (id,popis,mj,pocet,cenamj,sazbaid,dph,celkem,bezdph,fakturaid) VALUES (?,?,?,?,?,?,?,?,?,?)");
        q.addBindValue(dxutils::getLastID("polozka"));
        q.addBindValue(m_popis->text());
        q.addBindValue(m_mj->text());
        q.addBindValue(m_pocet->value());
        q.addBindValue(m_cenamj->value());
        q.addBindValue(m_sazba->itemData(m_sazba->currentIndex()));
        q.addBindValue(m_dph);
        q.addBindValue(m_celkem);
        q.addBindValue(m_bezdph->isChecked());
        q.addBindValue(m_faktura);
        if(!q.exec())
            QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
    }
    QDialog::accept();
}

int PolozkaDialog::id()
{
    return m_id;
}

void PolozkaDialog::slotUpdateDphCelkem()
{
    double sazba = m_platce?sazbaByID(m_sazba->itemData(m_sazba->currentIndex()).toInt()):0.0;
    double pocet = m_pocet->value();
    double cenamj = m_cenamj->value();
    if(m_bezdph->isChecked()) m_dph = pocet*cenamj*(sazba/100.0);
    else
    {
        m_dph = pocet*cenamj*ceil4(sazba/(100.0+sazba));
    }
    m_labelDph->setText(QString("%L1").arg(m_dph,0,'f',2));
    m_celkem = (m_bezdph->isChecked()? m_dph : 0.0) + cenamj*pocet;
    m_labelCelkem->setText(QString("%L1").arg(m_celkem,0,'f',2));
}

void PolozkaDialog::slotAddSazba()
{
    bool ok;
    int sazba = QInputDialog::getInt(this, "Sazba DPH", "Zadejte novou sazbu", 1, 1, 99, 1, &ok);
    if(ok)
    {
        ok = true;
        QSqlQuery q(QString("SELECT sazba FROM dphsazby WHERE rok=%1 ORDER BY sazba DESC").arg(rokFaktury()));
        while(q.next())
        {
            if(sazba == q.value(1).toInt())
            {
                ok = false;
                break;
            }
        }
        if(ok)
        {
            int id = dxutils::getLastID("dphsazby");
            q.prepare("INSERT INTO dphsazby (id, sazba, rok) VALUES (?,?,?)");
            q.addBindValue(id);
            q.addBindValue(sazba);
            q.addBindValue(rokFaktury());
            if(!q.exec())
                QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
            else
            {
                m_sazba->clear();
                q.exec(QString("SELECT id,sazba FROM dphsazby WHERE rok=%1 ORDER BY sazba DESC").arg(rokFaktury()));
                while(q.next())
                {
                    m_sazba->addItem(QString("%1%").arg(q.value(1).toInt()), q.value(0).toInt());
                }
                m_sazba->addItem("0%", 0);
            }
        }
    }
}

double PolozkaDialog::sazbaByID(int id)
{
    QSqlQuery q(QString("SELECT sazba FROM dphsazby WHERE id=%1").arg(id));
    if(q.first())
        return q.value(0).toDouble();
    else
        return 0.0; //id=0 je nulova sazba DPH
}

int PolozkaDialog::rokFaktury()
{
    if(m_edit)
    {
        QSqlQuery q(QString("SELECT rok FROM faktura WHERE id=%1").arg(m_faktura));
        if(q.first())
            return q.value(0).toInt();
    }
    return preferences.m_rok;
}


