/***************************************************************
 * Name:      previewdialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <QtSql>
#include "previewdialog.h"
#include "dxutils.h"
#include "preferences.h"

const QString pfakturaTags[] = {"{faktura-cislo}","{faktura-celkem}","{faktura-uhrada}","{faktura-banka}",
                               "{faktura-vystaveni}","{faktura-splatnost}","{faktura-duzp}",
                               "{faktura-ks}","{faktura-vs}","{faktura-mena}"};

const int numpFakturaTags = (sizeof(pfakturaTags)/sizeof(pfakturaTags[0]));

bool isRightpFakturaTag(const QString &line)
{
    for(int i=0; i<numpFakturaTags; i++)
    {
        if(line.contains(pfakturaTags[i]))
            return true;
    }
    return false;
}

const QString pfirmaTags[] = {"{firma-nazev}","{firma-ulice}","{firma-registrace}","{firma-logo}","{firma-razitko}",
                             "{firma-psc}","{firma-misto}","{firma-zeme}","{firma-ic}","{firma-dic}",
                             "{firma-telefon}","{firma-email}","{firma-www}"};

const int numpFirmaTags = (sizeof(pfirmaTags)/sizeof(pfirmaTags[0]));

bool isRightpFirmaTag(const QString &line)
{
    for(int i=0; i<numpFirmaTags; i++)
    {
        if(line.contains(pfirmaTags[i]))
            return true;
    }
    return false;
}

const QString padresarTags[] = {"{adresar-nazev}","{adresar-ulice}","{adresar-psc}","{adresar-misto}",
                             "{adresar-zeme}","{adresar-ic}","{adresar-dic}"};

const int numpAdresarTags = (sizeof(padresarTags)/sizeof(padresarTags[0]));

bool isRightpAdresarTag(const QString &line)
{
    for(int i=0; i<numpAdresarTags; i++)
    {
        if(line.contains(padresarTags[i]))
            return true;
    }
    return false;
}

const QString ppolozkaTags[] = {"{polozka-popis}","{polozka-pocet}","{polozka-cenamj}",
                             "{polozka-sazba}","{polozka-dph}","{polozka-celkem}"};

const int numpPolozkaTags = (sizeof(ppolozkaTags)/sizeof(ppolozkaTags[0]));

bool isRightpPolozkaTag(const QString &line)
{
    for(int i=0; i<numpPolozkaTags; i++)
    {
        if(line.contains(ppolozkaTags[i]))
            return true;
    }
    return false;
}

const QString psazbaTags[] = {"{dph-sazba}","{dph-zaklad}","{dph-dph}"};

const int numpSazbaTags = (sizeof(psazbaTags)/sizeof(psazbaTags[0]));

bool isRightpSazbaTag(const QString &line)
{
    for(int i=0; i<numpSazbaTags; i++)
    {
        if(line.contains(psazbaTags[i]))
            return true;
    }
    return false;
}

PreviewDialog::PreviewDialog(const QString &file, QWidget *parent) :
    QDialog(parent), m_file(file)
{
    m_platce = false;
    if(m_file.contains("{polozka-dph}"))
        m_platce = true;
    QDialogButtonBox *buttons;
    buttons = new QDialogButtonBox(QDialogButtonBox::Ok);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    m_faktura = new QWebView;
    createText();
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(m_faktura);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle(QString::fromUtf8("Náhled šablony"));
}

QString PreviewDialog::parseFakturaLine(QString line)
{
    QDate current = QDate::currentDate();
    QDate splatnost = current.addDays(preferences.m_splatnost);
    if(line.contains("{faktura-cislo}"))
    {
        line = line.replace("{faktura-cislo}","2014000001");
    }
    if(line.contains("{faktura-celkem}"))
    {
        line = line.replace("{faktura-celkem}",QString("%L1").arg(336.0, 0,'f',2));
    }
    if(line.contains("{faktura-vystaveni}"))
    {
        line = line.replace("{faktura-vystaveni}",current.toString("dd.MM.yyyy"));
    }
    if(line.contains("{faktura-splatnost}"))
    {
        line = line.replace("{faktura-splatnost}",splatnost.toString("dd.MM.yyyy"));
    }
    if(line.contains("{faktura-duzp}"))
    {
        line = line.replace("{faktura-duzp}",current.toString("dd.MM.yyyy"));
    }
    if(line.contains("{faktura-uhrada}"))
    {
        line = line.replace("{faktura-uhrada}",QString::fromUtf8("Převodem"));
    }
    if(line.contains("{faktura-mena}"))
    {
        line = line.replace("{faktura-mena}",QString::fromUtf8("Kč"));
    }
    if(line.contains("{faktura-banka}"))
    {
        line.replace("{faktura-banka}","100256368/0333");
    }
    if(line.contains("{faktura-vs}"))
    {
        line.replace("{faktura-vs}","2014000001");
    }
    if(line.contains("{faktura-ks}"))
    {
        line.replace("{faktura-ks}","0308");
    }
    return line;
}

QString PreviewDialog::parseFirmaLine(QString line)
{
    QSqlQuery firma(QString("SELECT * FROM firma_view WHERE id = %1").arg(preferences.m_firmaid));
    firma.first();
    if(line.contains("{firma-nazev}"))
    {
        line.replace("{firma-nazev}",firma.value(firma.record().indexOf("nazev")).toString());
    }
    if(line.contains("{firma-ulice}"))
    {
        line.replace("{firma-ulice}",QString("%1 %2").arg(firma.value(firma.record().indexOf("ulice")).toString())
                            .arg(firma.value(firma.record().indexOf("cp")).toString()));
    }
    if(line.contains("{firma-misto}"))
    {
        line.replace("{firma-misto}",firma.value(firma.record().indexOf("misto")).toString());
    }
    if(line.contains("{firma-psc}"))
    {
        line.replace("{firma-psc}",firma.value(firma.record().indexOf("psc")).toString());
    }
    if(line.contains("{firma-zeme}"))
    {
        line.replace("{firma-zeme}",firma.value(firma.record().indexOf("zeme")).toString());
    }
    if(line.contains("{firma-registrace}"))
    {
        line.replace("{firma-registrace}",firma.value(firma.record().indexOf("registrace")).toString());
    }
    if(line.contains("{firma-ic}"))
    {
        line.replace("{firma-ic}",firma.value(firma.record().indexOf("ic")).toString());
    }
    if(line.contains("{firma-dic}"))
    {
        line.replace("{firma-dic}",firma.value(firma.record().indexOf("dic")).toString());
    }
    if(line.contains("{firma-telefon}"))
    {
        line.replace("{firma-telefon}",firma.value(firma.record().indexOf("telefon")).toString());
    }
    if(line.contains("{firma-email}"))
    {
        line.replace("{firma-email}",firma.value(firma.record().indexOf("email")).toString());
    }
    if(line.contains("{firma-www}"))
    {
        line.replace("{firma-www}",firma.value(firma.record().indexOf("www")).toString());
    }
    if(line.contains("{firma-logo}"))
    {
        QString logo = firma.value(firma.record().indexOf("logo")).toString();
        if(logo.isEmpty()) line.replace("{firma-logo}","");
        else line.replace("{firma-logo}",QString("<br><img src=\"file:////%1\" />").arg(logo));
    }
    if(line.contains("{firma-razitko}"))
    {
        QString razitko = firma.value(firma.record().indexOf("razitko")).toString();
        if(razitko.isEmpty()) line.replace("{firma-razitko}","<br><br><br><br><br><br><br><br><br>");
        else line.replace("{firma-razitko}",QString("<br><img src=\"file:////%1\" /><br>").arg(razitko));
    }
    return line;
}

QString PreviewDialog::parseAdresarLine(QString line)
{
    if(line.contains("{adresar-nazev}"))
    {
        line.replace("{adresar-nazev}","Novak s.r.o.");
    }
    if(line.contains("{adresar-ulice}"))
    {
        line.replace("{adresar-ulice}","Jana Masaryka 152");
    }
    if(line.contains("{adresar-misto}"))
    {
        line.replace("{adresar-misto}","Novakov");
    }
    if(line.contains("{adresar-psc}"))
    {
        line.replace("{adresar-psc}","153 13");
    }
    if(line.contains("{adresar-zeme}"))
    {
        line.replace("{adresar-zeme}",QString::fromUtf8("ČR"));
    }
    if(line.contains("{adresar-ic}"))
    {
        line.replace("{adresar-ic}","71285013");
    }
    if(line.contains("{adresar-dic}"))
    {
        line.replace("{adresar-dic}","CZ71285013");
    }
    return line;
}

QString PreviewDialog::parsePolozkaLine(QString line)
{
    if(line.contains("{polozka-dph}"))
    {
        QString text;
        text += QString("%L1<br>").arg(21.0, 0,'f',2);
        text += QString("%L1<br>").arg(15.0, 0,'f',2);
        text += QString("%L1<br>").arg(0.0, 0,'f',2);
        line.replace("{polozka-dph}",text);
    }
    if(line.contains("{polozka-popis}"))
    {
        QString text;
        text += QString("%1<br>").arg(QString::fromUtf8("Položka č.1"));
        text += QString("%1<br>").arg(QString::fromUtf8("Položka č.2"));
        text += QString("%1<br>").arg(QString::fromUtf8("Položka č.3"));
        line.replace("{polozka-popis}",text);
    }
    if(line.contains("{polozka-pocet}"))
    {
        QString text;
        text += QString("%L1 ks<br>").arg(1.0, 0,'f',2);
        text += QString("%L1 ks<br>").arg(1.0, 0,'f',2);
        text += QString("%L1 ks<br>").arg(1.0, 0,'f',2);
        line.replace("{polozka-pocet}",text);
    }
    if(line.contains("{polozka-cenamj}"))
    {
        QString text;
        if(m_platce)
        {
            text += QString("%L1<br>").arg(100.0, 0,'f',2);
            text += QString("%L1<br>").arg(100.0, 0,'f',2);
            text += QString("%L1<br>").arg(100.0, 0,'f',2);
        }
        else
        {
            text += QString("%L1<br>").arg(121.0, 0,'f',2);
            text += QString("%L1<br>").arg(115.0, 0,'f',2);
            text += QString("%L1<br>").arg(100.0, 0,'f',2);
        }
        line.replace("{polozka-cenamj}",text);
    }
    if(line.contains("{polozka-sazba}"))
    {
        QString text;
        text += QString("%1<br>").arg(21);
        text += QString("%1<br>").arg(15);
        text += QString("%1<br>").arg(0);
        line.replace("{polozka-sazba}",text);
    }
    if(line.contains("{polozka-celkem}"))
    {
        QString text;
        text += QString("%L1<br>").arg(121.0, 0,'f',2);
        text += QString("%L1<br>").arg(115.0, 0,'f',2);
        text += QString("%L1<br>").arg(100.0, 0,'f',2);
        line.replace("{polozka-celkem}",text);
    }
    return line;
}

QString PreviewDialog::parseSazbaLine(QString line)
{
    QString sazba, zaklad, dph;
    zaklad += QString("%L1<br>").arg(100.0,0,'f',2);
    dph += QString("%L1<br>").arg(21.0,0,'f',2);
    sazba += QString("%1%<br>").arg(21);
    zaklad += QString("%L1<br>").arg(100.0,0,'f',2);
    dph += QString("%L1<br>").arg(15.0,0,'f',2);
    sazba += QString("%1%<br>").arg(15);
    zaklad += QString("%L1<br>").arg(100.0,0,'f',2);
    dph += QString("%L1<br>").arg(0.0,0,'f',2);
    sazba += QString("%1%<br>").arg(0);
    line.replace("{dph-zaklad}", zaklad);
    line.replace("{dph-dph}", dph);
    line.replace("{dph-sazba}", sazba);
    return line;
}

void PreviewDialog::createText()
{
    QString text;
    QApplication::setOverrideCursor(Qt::WaitCursor);
    QStringList file = m_file.split('\n');
    foreach(QString line, file)
    {
        if(isRightpFakturaTag(line))
        {
            line = parseFakturaLine(line);
        }
        if(isRightpFirmaTag(line))
        {
            line = parseFirmaLine(line);
        }
        if(isRightpAdresarTag(line))
        {
            line = parseAdresarLine(line);
        }
        if(isRightpPolozkaTag(line))
        {
            line = parsePolozkaLine(line);
        }
        if(isRightpSazbaTag(line))
        {
            line = parseSazbaLine(line);
        }
        text += line;
    }
    m_faktura->setHtml(text);
    QApplication::restoreOverrideCursor();
}

