/***************************************************************
 * Name:      printfadialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include <QPrinter>
#include <QPrintDialog>
#include "printfadialog.h"
#include "utils/faparser.h"
#include "dxutils.h"
#include "preferences.h"

PrintfaDialog::PrintfaDialog(int id, QWidget *parent) :
    QDialog(parent), m_id(id)
{
    QSqlQuery faktura(QString("SELECT platce FROM faktura WHERE id = %1").arg(m_id));
    faktura.first();
    m_platce = faktura.value(0).toBool();
    QHBoxLayout *print = new QHBoxLayout;
    m_internal = new QPushButton(QString::fromUtf8("Použít výchozí šablonu"));
    connect(m_internal, SIGNAL(clicked()), this, SLOT(slotInternal()));
    print->addWidget(m_internal);
    m_custom = new QPushButton(QString::fromUtf8("Použít vlastní šablonu"));
    connect(m_custom, SIGNAL(clicked()), this, SLOT(slotCustom()));
    print->addWidget(m_custom);
    m_print = new QPushButton("&Tisk");
    connect(m_print, SIGNAL(clicked()), this, SLOT(slotPrint()));
    print->addWidget(m_print);
    m_printPdf = new QPushButton(QString::fromUtf8("Uložit jako &PDF"));
    connect(m_printPdf, SIGNAL(clicked()), this, SLOT(slotPrintPDF()));
    print->addWidget(m_printPdf);
    print->addStretch();
    QDialogButtonBox *buttons;
    buttons = new QDialogButtonBox(QDialogButtonBox::Ok);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    m_faktura = new QWebView;
    createText();
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addLayout(print);
    mainLayout->addWidget(m_faktura);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle("Tisk faktury");
}

void PrintfaDialog::slotPrint()
{
    QPrinter printer;
    QPrintDialog *dialog = new QPrintDialog(&printer);
    if(dialog->exec() == QDialog::Accepted)
        m_faktura->print(&printer);
}

void PrintfaDialog::slotPrintPDF()
{
    QString filename = QFileDialog::getSaveFileName(this, QString::fromUtf8("Uložit jako PDF"), QString(), "*.pdf");
    if(!filename.isEmpty())
    {
        if(QFileInfo(filename).suffix().isEmpty()) filename.append(".pdf");
        QPrinter printer;
        printer.setOutputFormat(QPrinter::PdfFormat);
        printer.setOutputFileName(filename);
        m_faktura->print(&printer);
    }
}

void PrintfaDialog::createText()
{
    QString filename = selectTemplate();
    QFile file(filename);
    if(!file.open(QFile::ReadOnly | QFile::Text))
    {
        m_faktura->setHtml(QString::fromUtf8("<html><head></head><body>Nemohu otevřít šablonu: %1</body></html>").arg(filename));
        return;
    }
    QApplication::setOverrideCursor(Qt::WaitCursor);
    FaParser parser(m_id);
    m_faktura->setHtml(parser.parseFile(filename));
    QApplication::restoreOverrideCursor();
}

QString PrintfaDialog::selectTemplate()
{
    QString temp = "";
    QSqlQuery q(QString("SELECT vydana,typid FROM faktura WHERE id=%1").arg(m_id));
    q.first();
    if(q.value(0).toBool())
    {
        switch(q.value(1).toInt())
        {
            case 1:
            {
                if(m_platce)
                {
                    temp = preferences.m_favydanaplatce;
                    if(temp == "internal")
                    {
                        m_internal->hide();
                        return ":/templates/favydanaplatce.html";
                    }
                    else return temp;
                }
                else
                {
                    temp = preferences.m_favydananeplatce;
                    if(temp == "internal")
                    {
                        m_internal->hide();
                        return ":/templates/favydananeplatce.html";
                    }
                    else return temp;
                }
            }
            default:
            {
                if(m_platce)
                {
                    temp = preferences.m_zalohavydanaplatce;
                    if(temp == "internal")
                    {
                        m_internal->hide();
                        return ":/templates/zalohavydanaplatce.html";
                    }
                    else return temp;
                }
                else
                {
                    temp = preferences.m_zalohavydananeplatce;
                    if(temp == "internal")
                    {
                        m_internal->hide();
                        return ":/templates/zalohavydananeplatce.html";
                    }
                    else return temp;
                }
            }
        }
    }
    else
    {
        switch(q.value(1).toInt())
        {
            case 1:
            {
                if(m_platce)
                {
                    temp = preferences.m_faprijataplatce;
                    if(temp == "internal")
                    {
                        m_internal->hide();
                        return ":/templates/faprijataplatce.html";
                    }
                    else return temp;
                }
                else
                {
                    temp = preferences.m_faprijataneplatce;
                    if(temp == "internal")
                    {
                        m_internal->hide();
                        return ":/templates/faprijataneplatce.html";
                    }
                    else return temp;
                }
            }
            default:
            {
                if(m_platce)
                {
                    temp = preferences.m_zalohaprijataplatce;
                    if(temp == "internal")
                    {
                        m_internal->hide();
                        return ":/templates/zalohaprijataplatce.html";
                    }
                    else return temp;
                }
                else
                {
                    temp = preferences.m_zalohaprijataneplatce;
                    if(temp == "internal")
                    {
                        m_internal->hide();
                        return ":/templates/zalohaprijataneplatce.html";
                    }
                    else return temp;
                }
            }
        }
    }
    return temp;
}

void PrintfaDialog::slotInternal()
{
    QSqlQuery q(QString("SELECT vydana,typid FROM faktura WHERE id=%1").arg(m_id));
    q.first();
    if(q.value(0).toBool())
    {
        switch(q.value(1).toInt())
        {
            case 1:
            {
                if(m_platce)
                {
                    preferences.m_favydanaplatce = "internal";
                }
                else
                {
                    preferences.m_favydananeplatce = "internal";
                }
            }
            default:
            {
                if(m_platce)
                {
                    preferences.m_zalohavydanaplatce = "internal";
                }
                else
                {
                    preferences.m_zalohavydananeplatce = "internal";
                }
            }
        }
    }
    else
    {
        switch(q.value(1).toInt())
        {
            case 1:
            {
                if(m_platce)
                {
                    preferences.m_faprijataplatce = "internal";
                }
                else
                {
                    preferences.m_faprijataneplatce = "internal";
                }
            }
            default:
            {
                if(m_platce)
                {
                    preferences.m_zalohaprijataplatce = "internal";
                }
                else
                {
                    preferences.m_zalohaprijataneplatce = "internal";
                }
            }
        }
    }
    m_internal->hide();
    createText();
    updateSettings();
}

void PrintfaDialog::slotCustom()
{
#if (QT_VERSION > 0x050000)
    QString filename = QFileDialog::getOpenFileName(this, QString::fromUtf8("Vyberte šablonu"), QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0), QString::fromUtf8("Šablona (*htm *html)"));
#else
    QString filename = QFileDialog::getOpenFileName(this, QString::fromUtf8("Vyberte šablonu"), QDesktopServices::storageLocation(QDesktopServices::HomeLocation), QString::fromUtf8("Šablona (*htm *html)"));
#endif
    if(!filename.isEmpty())
    {
        QSqlQuery q(QString("SELECT vydana,typid FROM faktura WHERE id=%1").arg(m_id));
        q.first();
        if(q.value(0).toBool())
        {
            switch(q.value(1).toInt())
            {
                case 1:
                {
                    if(m_platce)
                    {
                        preferences.m_favydanaplatce = filename;
                    }
                    else
                    {
                        preferences.m_favydananeplatce = filename;
                    }
                }
                default:
                {
                    if(m_platce)
                    {
                        preferences.m_zalohavydanaplatce = filename;
                    }
                    else
                    {
                        preferences.m_zalohavydananeplatce = filename;
                    }
                }
            }
        }
        else
        {
            switch(q.value(1).toInt())
            {
                case 1:
                {
                    if(m_platce)
                    {
                        preferences.m_faprijataplatce = filename;
                    }
                    else
                    {
                        preferences.m_faprijataneplatce = filename;
                    }
                }
                default:
                {
                    if(m_platce)
                    {
                        preferences.m_zalohaprijataplatce = filename;
                    }
                    else
                    {
                        preferences.m_zalohaprijataneplatce = filename;
                    }
                }
            }
        }
    }
    m_internal->show();
    createText();
    updateSettings();
}

void PrintfaDialog::updateSettings()
{
    QSqlQuery q;
    q.prepare("UPDATE settings SET favydanaplatce=?,favydananeplatce=?,zalohavydanaplatce=?,zalohavydananeplatce=?,"
              "faprijataplatce=?,faprijataneplatce=?,zalohaprijataplatce=?,zalohaprijataneplatce=? WHERE firmaid=?");
    q.addBindValue(preferences.m_favydanaplatce);
    q.addBindValue(preferences.m_favydananeplatce);
    q.addBindValue(preferences.m_zalohavydanaplatce);
    q.addBindValue(preferences.m_zalohavydananeplatce);
    q.addBindValue(preferences.m_faprijataplatce);
    q.addBindValue(preferences.m_faprijataneplatce);
    q.addBindValue(preferences.m_zalohaprijataplatce);
    q.addBindValue(preferences.m_zalohaprijataneplatce);
    q.addBindValue(preferences.m_firmaid);
    if(!q.exec())
        QMessageBox::warning(this, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
}

