/***************************************************************
 * Name:      printprijmyvydajedialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include <QPrinter>
#include <QPrintDialog>
#include "printprijmyvydajedialog.h"
#include "dxutils.h"
#include "preferences.h"

const QString prijmyTags[] = {"{firma}","{rok}","{prijmy}","{vydane}","{vydaje}","{prijate}"};

const int numPrijmyTags = (sizeof(prijmyTags)/sizeof(prijmyTags[0]));

bool isRightPrijmyTag(const QString &line)
{
    for(int i=0; i<numPrijmyTags; i++)
    {
        if(line.contains(prijmyTags[i]))
            return true;
    }
    return false;
}

PrintPrijmyVydajeDialog::PrintPrijmyVydajeDialog(QWidget *parent) :
    QDialog(parent)
{
    m_platce = dxutils::isPlatce();
    m_doklady = new QCheckBox("Tisknout i doklady");
    m_doklady->setChecked(false);
    connect(m_doklady, SIGNAL(stateChanged(int)), this, SLOT(slotDoklady()));
    QHBoxLayout *print = new QHBoxLayout;
    m_print = new QPushButton("&Tisk");
    connect(m_print, SIGNAL(clicked()), this, SLOT(slotPrint()));
    print->addWidget(m_print);
    print->addStretch();
    QDialogButtonBox *buttons;
    buttons = new QDialogButtonBox(QDialogButtonBox::Ok);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    m_seznam = new QWebView;
    createText();
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(m_doklady);
    mainLayout->addLayout(print);
    mainLayout->addWidget(m_seznam);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle(QString::fromUtf8("Tisk příjmů a výdajů"));
}

void PrintPrijmyVydajeDialog::slotPrint()
{
    QPrinter printer;
    QPrintDialog *dialog = new QPrintDialog(&printer);
    if(dialog->exec() == QDialog::Accepted)
        m_seznam->print(&printer);
}

void PrintPrijmyVydajeDialog::slotDoklady()
{
    QApplication::setOverrideCursor(Qt::WaitCursor);
    createText();
    QApplication::restoreOverrideCursor();
}

void PrintPrijmyVydajeDialog::createText()
{
    QString line, text;
    QApplication::setOverrideCursor(Qt::WaitCursor);
    QFile file(m_platce?":/templates/prijmyvydajeplatce.html":":/templates/prijmyvydajeneplatce.html");
    if(file.open(QFile::ReadOnly | QFile::Text))
    {
        while(!file.atEnd())
        {
            line = QString::fromUtf8(file.readLine());
            if(isRightPrijmyTag(line))
            {
                line = parseLine(line);
            }
            text += line;
        }
    }
    m_seznam->setHtml(text);
    QApplication::restoreOverrideCursor();
}

QString PrintPrijmyVydajeDialog::parseLine(QString line)
{
    if(line.contains("{firma}"))
    {
        QSqlQuery q(QString("SELECT nazev FROM firma WHERE id=%1").arg(preferences.m_firmaid));
        q.first();
        line.replace("{firma}",q.value(0).toString());
    }
    if(line.contains("{rok}"))
    {
        line.replace("{rok}",QString::number(preferences.m_rok));
    }
    if(line.contains("{prijmy}"))
    {
        QSqlQuery q(QString("SELECT count() FROM faktura_view WHERE vydana=1 AND firmaid=%1 AND zaplaceno LIKE '%%2'")
               .arg(preferences.m_firmaid)
               .arg(preferences.m_rok));
        q.first();
        if(q.value(0).toBool())
        {
            int i = 0;
            q.exec(QString("SELECT sum(celkem),zkratka FROM faktura_view WHERE vydana=1 AND firmaid=%1 AND zaplaceno LIKE '%%2' GROUP BY zkratka")
                   .arg(preferences.m_firmaid)
                   .arg(preferences.m_rok));
            while(q.next())
            {
                if(!i)
                {
                    line = QString::fromUtf8("<tr><td colspan=9><font size=3><b>Příjmy:</b></font></td></tr>");
                }
                line += QString("<tr><td></td>"
                                "<td align=\"right\">%1</td>"
                                "<td>%2</td>"
                                "</tr>")
                         .arg(QString("%L1").arg(q.value(0).toDouble(),0,'f',2))
                         .arg(q.value(1).toString());
                i++;
            }
        }
        else
        {
            line = QString::fromUtf8("<tr><td colspan=9><font size=3><b>Příjmy:</b></font></td></tr>");
            line += QString::fromUtf8("<tr><td colspan=9>Zatím žádné příjmy<td><tr>");
        }
    }
    if(line.contains("{vydane}"))
    {
        if(!m_doklady->isChecked()) return "";
        line = QString::fromUtf8("<tr><td colspan=9><font size=3><b>Vydané faktury:</b></font></td></tr>");
        QSqlQuery q(QString("SELECT count() FROM faktura_view WHERE vydana=1 AND firmaid=%1 AND zaplaceno LIKE '%%2'")
               .arg(preferences.m_firmaid)
               .arg(preferences.m_rok));
        q.first();
        if(q.value(0).toBool())
        {
            int size = q.value(0).toInt();
            line += "<tr>";
            line += "<td style=\"border-top: 3px solid #000000; border-left: 3px solid #000000\" align=\"center\">Typ</td>";
            line += QString::fromUtf8("<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Číslo</td>");
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Popis</td>";
            line += QString::fromUtf8("<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Odběratel</td>");
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Celkem</td>";
            line += QString::fromUtf8("<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Měna</td>");
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Vystavena</td>";
            if(m_platce) line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">DUZP</td>";
            line += QString::fromUtf8("<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Splatná</td>");
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000; border-right: 3px solid #000000\" align=\"center\">Zaplacena</td>";
            line += "</tr>";
            int i = 1;
            q.exec(QString("SELECT * FROM faktura_view WHERE vydana=1 AND firmaid=%1 AND zaplaceno LIKE '%%2' ORDER BY date(substr(zaplaceno,7,4)||'-'||substr(zaplaceno,4,2)||'-'||substr(zaplaceno,1,2))")
                   .arg(preferences.m_firmaid)
                   .arg(preferences.m_rok));
            while(q.next())
            {
                line += "<tr>";
                if(i == size)
                {
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 3px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(1).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(2).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"left\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(3).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(4).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"right\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(QString("%L1").arg(q.value(5).toDouble(),0,'f',2));
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(6).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(7).toString());
                    if(m_platce)
                    {
                        line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                                .arg(i==1?2:1)
                                .arg(q.value(8).toString());
                    }
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(9).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000; border-right: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(10).toString());
                }
                else
                {
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(1).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(2).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"left\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(3).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(4).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"right\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(QString("%L1").arg(q.value(5).toDouble(),0,'f',2));
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(6).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(7).toString());
                    if(m_platce)
                    {
                        line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                                .arg(i==1?2:1)
                                .arg(q.value(8).toString());
                    }
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(9).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-right: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(10).toString());
                }
                line += "</tr>";
                i++;
            }
        }
        else
        {
            line += QString::fromUtf8("<tr><td colspan=9>Zatím neuhrazena žádná faktura<td><tr>");
        }
    }
    if(line.contains("{vydaje}"))
    {
        QSqlQuery q(QString("SELECT count() FROM faktura_view WHERE vydana=0 AND firmaid=%1 AND zaplaceno LIKE '%%2'")
               .arg(preferences.m_firmaid)
               .arg(preferences.m_rok));
        q.first();
        if(q.value(0).toBool())
        {
            int i = 0;
            q.exec(QString("SELECT sum(celkem),zkratka FROM faktura_view WHERE vydana=0 AND firmaid=%1 AND zaplaceno LIKE '%%2' GROUP BY zkratka")
                   .arg(preferences.m_firmaid)
                   .arg(preferences.m_rok));
            while(q.next())
            {
                if(!i)
                {
                    line = QString::fromUtf8("<tr><td colspan=9><font size=3><b>Výdaje:</b></font></td></tr>");
                }
                line += QString("<tr><td></td>"
                                "<td align=\"right\">%1</td>"
                                "<td>%2</td>"
                                "</tr>")
                         .arg(QString("%L1").arg(q.value(0).toDouble(),0,'f',2))
                         .arg(q.value(1).toString());
                i++;
            }
        }
        else
        {
            line = QString::fromUtf8("<tr><td colspan=9><font size=3><b>Výdaje:</b></font></td></tr>");
            line += QString::fromUtf8("<tr><td colspan=9>Zatím žádné výdaje<td><tr>");
        }
    }
    if(line.contains("{prijate}"))
    {
        if(!m_doklady->isChecked()) return "";
        line = QString::fromUtf8("<tr><td colspan=9><font size=3><b>Přijaté faktury:</b></font></td></tr>");
        QSqlQuery q(QString("SELECT count() FROM faktura_view WHERE vydana=0 AND firmaid=%1 AND zaplaceno LIKE '%%2'")
               .arg(preferences.m_firmaid)
               .arg(preferences.m_rok));
        q.first();
        if(q.value(0).toBool())
        {
            int size = q.value(0).toInt();
            line += "<tr>";
            line += "<td style=\"border-top: 3px solid #000000; border-left: 3px solid #000000\" align=\"center\">Typ</td>";
            line += QString::fromUtf8("<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Číslo</td>");
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Popis</td>";
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Dodavatel</td>";
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Celkem</td>";
            line += QString::fromUtf8("<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Měna</td>");
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Vystavena</td>";
            if(m_platce) line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">DUZP</td>";
            line += QString::fromUtf8("<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Splatná</td>");
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000; border-right: 3px solid #000000\" align=\"center\">Zaplacena</td>";
            line += "</tr>";
            int i = 1;
            q.exec(QString("SELECT * FROM faktura_view WHERE vydana=0 AND firmaid=%1 AND zaplaceno LIKE '%%2' ORDER BY date(substr(zaplaceno,7,4)||'-'||substr(zaplaceno,4,2)||'-'||substr(zaplaceno,1,2))")
                   .arg(preferences.m_firmaid)
                   .arg(preferences.m_rok));
            while(q.next())
            {
                line += "<tr>";
                if(i == size)
                {
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 3px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(1).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(2).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"left\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(3).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(4).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"right\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(QString("%L1").arg(q.value(5).toDouble(),0,'f',2));
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(6).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(7).toString());
                    if(m_platce)
                    {
                        line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                                .arg(i==1?2:1)
                                .arg(q.value(8).toString());
                    }
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(9).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000; border-right: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(10).toString());
                }
                else
                {
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(1).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(2).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"left\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(3).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(4).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"right\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(QString("%L1").arg(q.value(5).toDouble(),0,'f',2));
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(6).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(7).toString());
                    if(m_platce)
                    {
                        line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                                .arg(i==1?2:1)
                                .arg(q.value(8).toString());
                    }
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(9).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-right: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(10).toString());
                }
                line += "</tr>";
                i++;
            }
        }
        else
        {
            line += QString::fromUtf8("<tr><td colspan=9>Zatím neuhrazena žádná faktura<td><tr>");
        }
    }
    return line;
}

