/***************************************************************
 * Name:      printseznamfadialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include <QPrinter>
#include <QPrintDialog>
#include "printseznamfadialog.h"
#include "dxutils.h"
#include "preferences.h"

const QString seznamTags[] = {"{firma}","{rok}","{vydane}","{prijate}"};

const int numSeznamTags = (sizeof(seznamTags)/sizeof(seznamTags[0]));

bool isRightSeznamTag(const QString &line)
{
    for(int i=0; i<numSeznamTags; i++)
    {
        if(line.contains(seznamTags[i]))
            return true;
    }
    return false;
}

PrintSeznamFaDialog::PrintSeznamFaDialog(QWidget *parent) :
    QDialog(parent)
{
    m_platce = dxutils::isPlatce();
    QHBoxLayout *print = new QHBoxLayout;
    m_print = new QPushButton("&Tisk");
    connect(m_print, SIGNAL(clicked()), this, SLOT(slotPrint()));
    print->addWidget(m_print);
    print->addStretch();
    QDialogButtonBox *buttons;
    buttons = new QDialogButtonBox(QDialogButtonBox::Ok);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    m_seznam = new QWebView;
    createText();
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addLayout(print);
    mainLayout->addWidget(m_seznam);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle("Tisk seznamu faktur");
}

void PrintSeznamFaDialog::slotPrint()
{
    QPrinter printer;
    QPrintDialog *dialog = new QPrintDialog(&printer);
    if(dialog->exec() == QDialog::Accepted)
        m_seznam->print(&printer);
}

void PrintSeznamFaDialog::createText()
{
    QString line, text;
    QApplication::setOverrideCursor(Qt::WaitCursor);
    QFile file(m_platce?":/templates/seznamfakturplatce.html":":/templates/seznamfakturneplatce.html");
    if(file.open(QFile::ReadOnly | QFile::Text))
    {
        while(!file.atEnd())
        {
            line = QString::fromUtf8(file.readLine());
            if(isRightSeznamTag(line))
            {
                line = parseLine(line);
            }
            text += line;
        }
    }
    m_seznam->setHtml(text);
    QApplication::restoreOverrideCursor();
}

QString PrintSeznamFaDialog::parseLine(QString line)
{
    if(line.contains("{firma}"))
    {
        QSqlQuery q(QString("SELECT nazev FROM firma WHERE id=%1").arg(preferences.m_firmaid));
        q.first();
        line.replace("{firma}",q.value(0).toString());
    }
    if(line.contains("{rok}"))
    {
        line.replace("{rok}",QString::number(preferences.m_rok));
    }
    if(line.contains("{vydane}"))
    {
        QSqlQuery q(QString("SELECT count() FROM faktura_view WHERE vydana=1 AND firmaid=%1 AND rok=%2")
               .arg(preferences.m_firmaid)
               .arg(preferences.m_rok));
        q.first();
        if(q.value(0).toBool())
        {
            int size = q.value(0).toInt();
            line = "<tr>";
            line += "<td style=\"border-top: 3px solid #000000; border-left: 3px solid #000000\" align=\"center\">Typ</td>";
            line += QString::fromUtf8("<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Číslo</td>");
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Popis</td>";
            line += QString::fromUtf8("<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Odběratel</td>");
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Celkem</td>";
            line += QString::fromUtf8("<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Měna</td>");
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Vystavena</td>";
            if(m_platce) line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">DUZP</td>";
            line += QString::fromUtf8("<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Splatná</td>");
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000; border-right: 3px solid #000000\" align=\"center\">Zaplacena</td>";
            line += "</tr>";
            int i = 1;
            q.exec(QString("SELECT * FROM faktura_view WHERE vydana=1 AND firmaid=%1 AND rok=%2")
                   .arg(preferences.m_firmaid)
                   .arg(preferences.m_rok));
            while(q.next())
            {
                line += "<tr>";
                if(i == size)
                {
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 3px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(1).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(2).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"left\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(3).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(4).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"right\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(QString("%L1").arg(q.value(5).toDouble(),0,'f',2));
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(6).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(7).toString());
                    if(m_platce)
                    {
                        line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                                .arg(i==1?2:1)
                                .arg(q.value(8).toString());
                    }
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(9).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000; border-right: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(10).toString());
                }
                else
                {
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(1).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(2).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"left\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(3).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(4).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"right\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(QString("%L1").arg(q.value(5).toDouble(),0,'f',2));
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(6).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(7).toString());
                    if(m_platce)
                    {
                        line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                                .arg(i==1?2:1)
                                .arg(q.value(8).toString());
                    }
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(9).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-right: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(10).toString());
                }
                line += "</tr>";
                i++;
            }
        }
        else
        {
            line = QString::fromUtf8("<tr><td colspan=9>Zatím nevydána žádná faktura<td><tr>");
        }
    }
    if(line.contains("{prijate}"))
    {
        QSqlQuery q(QString("SELECT count() FROM faktura_view WHERE vydana=0 AND firmaid=%1 AND rok=%2")
               .arg(preferences.m_firmaid)
               .arg(preferences.m_rok));
        q.first();
        if(q.value(0).toBool())
        {
            int size = q.value(0).toInt();
            line = "<tr>";
            line += "<td style=\"border-top: 3px solid #000000; border-left: 3px solid #000000\" align=\"center\">Typ</td>";
            line += QString::fromUtf8("<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Číslo</td>");
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Popis</td>";
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Dodavatel</td>";
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Celkem</td>";
            line += QString::fromUtf8("<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Měna</td>");
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Vystavena</td>";
            if(m_platce) line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">DUZP</td>";
            line += QString::fromUtf8("<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000\" align=\"center\">Splatná</td>");
            line += "<td style=\"border-top: 3px solid #000000; border-left: 1px solid #000000; border-right: 3px solid #000000\" align=\"center\">Zaplacena</td>";
            line += "</tr>";
            int i = 1;
            q.exec(QString("SELECT * FROM faktura_view WHERE vydana=0 AND firmaid=%1 AND rok=%2")
                   .arg(preferences.m_firmaid)
                   .arg(preferences.m_rok));
            while(q.next())
            {
                line += "<tr>";
                if(i == size)
                {
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 3px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(1).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(2).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"left\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(3).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(4).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"right\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(QString("%L1").arg(q.value(5).toDouble(),0,'f',2));
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(6).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(7).toString());
                    if(m_platce)
                    {
                        line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                                .arg(i==1?2:1)
                                .arg(q.value(8).toString());
                    }
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(9).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-bottom: 3px solid #000000; border-right: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(10).toString());
                }
                else
                {
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(1).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(2).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"left\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(3).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(4).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"right\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(QString("%L1").arg(q.value(5).toDouble(),0,'f',2));
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(6).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(7).toString());
                    if(m_platce)
                    {
                        line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                                .arg(i==1?2:1)
                                .arg(q.value(8).toString());
                    }
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(9).toString());
                    line += QString("<td style=\"border-top: %1px solid #000000; border-left: 1px solid #000000; border-right: 3px solid #000000\" align=\"center\">%2</td>")
                            .arg(i==1?2:1)
                            .arg(q.value(10).toString());
                }
                line += "</tr>";
                i++;
            }
        }
        else
        {
            line = QString::fromUtf8("<tr><td colspan=9>Zatím nepřijata žádná faktura<td><tr>");
        }
    }
    return line;
}

