/***************************************************************
 * Name:      dxutils.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include <QtSql>
#include <QApplication>
#include <QFileInfo>
#include "dxutils.h"
#include "preferences.h"

dxutils::dxutils()
{
}

dxutils &dxutils::instance()
{
    static dxutils idxutils;
    return idxutils;
}

void dxutils::createDB()
{
    QSqlQuery query;

    query.exec("CREATE TABLE adresa (id INTEGER PRIMARY KEY AUTOINCREMENT, ulice TEXT, cp TEXT, misto TEXT, psc TEXT, zeme TEXT)");
    //Vlozeni defaultni adresy
    query.exec("INSERT INTO adresa (id) VALUES (0)");

    query.exec("CREATE TABLE adresar (id INTEGER PRIMARY KEY AUTOINCREMENT, nazev TEXT, ic TEXT, dic TEXT, email TEXT, public INTEGER, adresaid INTEGER,"
               "FOREIGN KEY(adresaid) REFERENCES adresa(id) ON UPDATE CASCADE)");
    //Vlozeni defaultniho odberatele/dodavatele
    query.exec("INSERT INTO adresar (id,adresaid,public) VALUES (0,0,0)");
    query.exec("CREATE VIEW adresar_view AS SELECT adresar.id,nazev,ic,dic,email,ulice,cp,misto,psc,zeme,public FROM adresar INNER JOIN adresa ON adresar.adresaid = adresa.id");
    //sqlite neumi update view, proto trigger :)
    query.exec("CREATE TRIGGER adresar_view_update INSTEAD OF UPDATE ON adresar_view BEGIN UPDATE adresar SET nazev = new.nazev,"
               "ic = new.ic, dic = new.dic, email = new.email WHERE id = new.id; UPDATE adresa SET ulice = new.ulice, cp = new.cp, misto = new.misto,"
               "psc = new.psc, zeme = new.zeme WHERE id = (SELECT adresaid FROM adresar WHERE id = new.id); END;");

    query.exec("CREATE TABLE firma (id INTEGER PRIMARY KEY AUTOINCREMENT, nazev TEXT, ic TEXT, dic TEXT, registrace TEXT, platce INTEGER, telefon TEXT,"
               "email TEXT, www TEXT, logo TEXT, razitko TEXT, adresaid INTEGER, FOREIGN KEY(adresaid) REFERENCES adresa(id) ON UPDATE CASCADE)");
    query.exec("CREATE VIEW firma_view AS SELECT firma.id,nazev,ic,dic,registrace,platce,telefon,email,www,logo,razitko,ulice,cp,misto,psc,zeme "
               "FROM firma INNER JOIN adresa ON firma.adresaid = adresa.id");
    //sqlite neumi update view, proto trigger :)
    query.exec("CREATE TRIGGER firma_view_update INSTEAD OF UPDATE ON firma_view BEGIN UPDATE firma SET nazev = new.nazev, ic = new.ic, dic = new.dic,"
               "registrace = new.registrace, platce = new.platce, telefon = new.telefon, email = new.email, www = new.www, logo = new.logo, razitko = new.razitko WHERE id = new.id;"
               " UPDATE adresa SET ulice = new.ulice, cp = new.cp, misto = new.misto, psc = new.psc, zeme = new.zeme WHERE id = (SELECT adresaid FROM firma WHERE id = new.id); END;");

    query.exec("CREATE TABLE settings (id INTEGER PRIMARY KEY AUTOINCREMENT, rok INTEGER, vydana TEXT, prijata TEXT, splatnost INTEGER,"
               "faprijataneplatce TEXT, faprijataplatce TEXT, favydananeplatce TEXT, favydanaplatce TEXT, zalohaprijataneplatce TEXT,"
               "zalohaprijataplatce TEXT, zalohavydananeplatce TEXT, zalohavydanaplatce TEXT,"
               "firmaid INTEGER, FOREIGN KEY(firmaid) REFERENCES firma(id) ON UPDATE CASCADE)");

    query.exec("CREATE TABLE smtp (id INTEGER PRIMARY KEY AUTOINCREMENT, server TEXT, port INTEGER, ssl INTEGER, autentifikace INTEGER,"
               "username TEXT, password TEXT, sender TEXT, subject TEXT, text TEXT,"
               "firmaid INTEGER, FOREIGN KEY(firmaid) REFERENCES firma(id) ON UPDATE CASCADE)");

    //Tabulky atd. pro fakturu
    query.exec("CREATE TABLE mena (id INTEGER PRIMARY KEY AUTOINCREMENT, zkratka TEXT)");
    query.exec(QString::fromUtf8("INSERT INTO mena (id,zkratka) VALUES (1,'Kč')"));
    query.exec("CREATE TABLE fatyp (id INTEGER PRIMARY KEY AUTOINCREMENT, typ TEXT)");
    query.exec("INSERT INTO fatyp (id,typ) VALUES (1,'Faktura')");
    query.prepare("INSERT INTO fatyp (id,typ) VALUES (2,?)");
    query.addBindValue(QString::fromUtf8("Zálohová faktura"));
    query.exec();
    query.exec("CREATE TABLE fauhrada (id INTEGER PRIMARY KEY AUTOINCREMENT, uhrada TEXT, hotove INTEGER NOT NULL)");
    query.prepare("INSERT INTO fauhrada (id,uhrada,hotove) VALUES (1,?,1)");
    query.addBindValue(QString::fromUtf8("Hotově"));
    query.exec();
    query.prepare("INSERT INTO fauhrada (id,uhrada,hotove) VALUES (2,?,0)");
    query.addBindValue(QString::fromUtf8("Převodem"));
    query.exec();
    query.exec("CREATE TABLE dphsazby (id INTEGER PRIMARY KEY AUTOINCREMENT, sazba INTEGER, rok INTEGER)");
    query.exec("INSERT INTO dphsazby (id,sazba) VALUES (0,0)");
    query.exec("INSERT INTO dphsazby (id,sazba,rok) VALUES (1,15,2013)");
    query.exec("INSERT INTO dphsazby (id,sazba,rok) VALUES (2,21,2013)");
    query.exec("INSERT INTO dphsazby (id,sazba,rok) VALUES (3,15,2014)");
    query.exec("INSERT INTO dphsazby (id,sazba,rok) VALUES (4,21,2014)");
    query.exec("INSERT INTO dphsazby (id,sazba,rok) VALUES (5,10,2015)");
    query.exec("INSERT INTO dphsazby (id,sazba,rok) VALUES (6,15,2015)");
    query.exec("INSERT INTO dphsazby (id,sazba,rok) VALUES (7,21,2015)");
    query.exec("CREATE TABLE banka (id INTEGER PRIMARY KEY AUTOINCREMENT, cislo TEXT, banka TEXT)");
    query.exec("INSERT INTO banka (id) VALUES (0)");
    query.exec("CREATE TABLE faktura (id INTEGER PRIMARY KEY AUTOINCREMENT,firmaid INTEGER,"
               "adresarid INTEGER DEFAULT 0,popis TEXT,last INTEGER,celkem REAL,cislo TEXT,vs TEXT,ss TEXT,ks TEXT,"
               "typid INTEGER NOT NULL,uhradaid INTEGER NOT NULL,bankaid INTEGER,menaid INTEGER DEFAULT 1,vydana INTEGER,"
               "platce INTEGER,vystaveni TEXT,duzp TEXT,splatnost TEXT,zaplaceno TEXT,rok INTEGER,"
               "FOREIGN KEY(firmaid) REFERENCES firma(id) ON UPDATE CASCADE ON DELETE CASCADE,"
               "FOREIGN KEY(adresarid) REFERENCES adresar(id) ON UPDATE CASCADE ON DELETE SET DEFAULT,"
               "FOREIGN KEY(typid) REFERENCES fatyp(id) ON UPDATE CASCADE,"
               "FOREIGN KEY(uhradaid) REFERENCES fauhrada(id) ON UPDATE CASCADE,"
               "FOREIGN KEY(bankaid) REFERENCES banka(id) ON UPDATE CASCADE,"
               "FOREIGN KEY(menaid) REFERENCES mena(id) ON UPDATE CASCADE)");
    query.exec("CREATE VIEW faktura_view AS SELECT faktura.id,typ,cislo,popis,adresar.nazev,celkem,mena.zkratka,vystaveni,duzp,splatnost,zaplaceno,rok,firma.id AS firmaid,vydana FROM faktura"
               " INNER JOIN firma ON faktura.firmaid = firma.id"
               " INNER JOIN adresar ON faktura.adresarid = adresar.id"
               " INNER JOIN fatyp ON faktura.typid = fatyp.id"
               " INNER JOIN mena ON faktura.menaid = mena.id");
    //bezpdh 1 znamena ze cenamj je bez dph, 0 opak
    query.exec("CREATE TABLE polozka (id INTEGER PRIMARY KEY AUTOINCREMENT, popis TEXT, mj TEXT, pocet REAL, cenamj REAL, sazbaid INTEGER, dph REAL, celkem REAL, bezdph INTEGER,"
               "fakturaid INTEGER,FOREIGN KEY(fakturaid) REFERENCES faktura(id) ON UPDATE CASCADE ON DELETE CASCADE,"
               "FOREIGN KEY(sazbaid) REFERENCES dphsazby(id) ON UPDATE CASCADE ON DELETE CASCADE)");
}

bool dxutils::checkPath(const QString &path)
{
    QFileInfo info(path);
    return info.exists();
}

bool dxutils::checkIC(const QString &ic)
{
    QRegExp rx("^\\d{8}$");
    if(rx.indexIn(ic)==-1) return false;
    int soucet = 0;
    for(int i=0; i<7; i++)
    {
        soucet += (ic[i].toLatin1()-48)*(8-i);
    }
    soucet = soucet % 11;
    int c;
    if(soucet == 0) c = 1;
    else if(soucet == 10) c = 1;
    else if(soucet == 1) c = 0;
    else c = 11 - soucet;
    return ic[7].toLatin1()-48 == c;
}

int dxutils::getLastID(const QString &table)
{
    QSqlQuery query;
    query.exec(QString("SELECT MAX(id) FROM %1").arg(table));
    if(query.first())
    {
        return query.value(0).toInt()+1;
    }
    return 1;
}

bool dxutils::isPlatce()
{
    QSqlQuery q;
    q.exec(QString("SELECT platce FROM firma WHERE id=%1").arg(preferences.m_firmaid));
    q.first();
    return q.value(0).toBool();
}


