/***************************************************************
 * Name:      faktury.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include <QVBoxLayout>
#include <QtSql>
#include "faktury.h"
#include "preferences.h"

Faktury::Faktury(QTreeWidgetItem *parent) :
    dxitem(parent)
{
    m_panel = new QWidget;
    m_view = new QTextBrowser;
    QVBoxLayout *mainFrame = new QVBoxLayout;
    mainFrame->addWidget(m_view);
    m_panel->setLayout(mainFrame);
    createInfo();
}

QWidget *Faktury::getPanel()
{
    createInfo();
    return m_panel;
}

void Faktury::firmaChanged()
{
    createInfo();
}

void Faktury::createInfo()
{
    QSqlQuery q(QString("SELECT nazev FROM firma WHERE id = %1").arg(preferences.m_firmaid));
    q.first();
    QString html = QString("<br><br><br><b>%1</b><br>").arg(q.value(0).toString());
    html.append("<br>");
    html.append(QString::fromUtf8("Účetní rok: <b>%1</b><br>").arg(preferences.m_rok));
    html.append("<br>");
    html.append(createVydane());
    html.append(createPrijate());
    m_view->setHtml(html);
}

QString Faktury::createVydane() const
{
    QString html = QString::fromUtf8("<b>Vydané faktury:</b><br>");
    QSqlQuery q(QString("SELECT count() FROM faktura_view WHERE vydana=1 AND firmaid=%1 AND rok=%2")
           .arg(preferences.m_firmaid)
           .arg(preferences.m_rok));
    q.first();
    if(q.value(0).toBool())
    {
        q.exec(QString("SELECT count(),sum(celkem),zkratka FROM faktura_view WHERE vydana=1 AND firmaid=%1 AND rok=%2 GROUP BY zkratka")
               .arg(preferences.m_firmaid)
               .arg(preferences.m_rok));
        while(q.next())
        {
            html.append(QString::fromUtf8("Vydáno faktur v %1: <b>%2</b><br>").arg(q.value(2).toString()).arg(q.value(0).toInt()));
            html.append(QString::fromUtf8("Za: <b>%L1 %2</b><br>").arg(q.value(1).toFloat(),0,'f',2).arg(q.value(2).toString()));
            QSqlQuery q1(QString("SELECT sum(celkem) FROM faktura_view WHERE vydana=1 AND firmaid=%1 AND rok=%2 AND zaplaceno NOT null AND zkratka='%3'")
                         .arg(preferences.m_firmaid)
                         .arg(preferences.m_rok)
                         .arg(q.value(2).toString()));
            q1.first();
            html.append(QString::fromUtf8("Z toho zaplaceno: <b>%L1 %2</b><br>").arg(q1.value(0).toFloat(),0,'f',2).arg(q.value(2).toString()));
            html.append("<br>");
        }
    }
    else
    {
        html.append(QString::fromUtf8("Zatím nevydána žádná faktura<br><br>"));
    }
    return html;
}

QString Faktury::createPrijate() const
{
    QString html = QString::fromUtf8("<b>Přijaté faktury:</b><br>");
    QSqlQuery q(QString("SELECT count() FROM faktura_view WHERE vydana=0 AND firmaid=%1 AND rok=%2")
           .arg(preferences.m_firmaid)
           .arg(preferences.m_rok));
    q.first();
    if(q.value(0).toBool())
    {
        q.exec(QString("SELECT count(),sum(celkem),zkratka FROM faktura_view WHERE vydana=0 AND firmaid=%1 AND rok=%2 GROUP BY zkratka")
               .arg(preferences.m_firmaid)
               .arg(preferences.m_rok));
        while(q.next())
        {
            html.append(QString::fromUtf8("Přijato faktur v %1: <b>%2</b><br>").arg(q.value(2).toString()).arg(q.value(0).toInt()));
            html.append(QString::fromUtf8("Za: <b>%L1 %2</b><br>").arg(q.value(1).toFloat(),0,'f',2).arg(q.value(2).toString()));
            QSqlQuery q1(QString("SELECT sum(celkem) FROM faktura_view WHERE vydana=0 AND firmaid=%1 AND rok=%2 AND zaplaceno NOT null AND zkratka='%3'")
                         .arg(preferences.m_firmaid)
                         .arg(preferences.m_rok)
                         .arg(q.value(2).toString()));
            q1.first();
            html.append(QString::fromUtf8("Z toho zaplaceno: <b>%L1 %2</b><br>").arg(q1.value(0).toFloat(),0,'f',2).arg(q.value(2).toString()));
            html.append("<br>");
        }
    }
    else
    {
        html.append(QString::fromUtf8("Zatím nepřijata žádná faktura<br><br>"));
    }
    return html;
}


