/***************************************************************
 * Name:      prijate.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include <QToolBar>
#include "prijate.h"
#include "dxutils.h"
#include "dialogs/fatypdialog.h"
#include "dialogs/fakturadialog.h"
#include "dialogs/printfadialog.h"
#include "dialogs/uhradadialog.h"
#include "preferences.h"

Prijate::Prijate(QTreeWidgetItem *parent) :
    dxitem(parent)
{
    setupModel();
    createActions();
    m_panel = new QWidget;
    QToolBar *faktura = new QToolBar(m_panel);
    faktura->setStyleSheet("QToolBar { border: 0px }");
    faktura->setMovable(false);
    faktura->addAction(m_addPrijata);
    faktura->addAction(m_editPrijata);
    faktura->addAction(m_deletePrijata);
    faktura->addSeparator();
    faktura->addAction(m_printPrijata);
    faktura->addSeparator();
    faktura->addAction(m_payPrijata);
    m_fakturyPrijate = new QTableView;
    m_fakturyPrijate->setModel(m_modelPrijate);
    m_fakturyPrijate->hideColumn(0);
    if(!dxutils::isPlatce()) m_fakturyPrijate->hideColumn(8);
    m_fakturyPrijate->hideColumn(11);
    m_fakturyPrijate->hideColumn(12);
    m_fakturyPrijate->hideColumn(13);
    m_fakturyPrijate->horizontalHeader()->setStretchLastSection(true);
    m_fakturyPrijate->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_fakturyPrijate->setSelectionMode(QAbstractItemView::SingleSelection);
    m_fakturyPrijate->verticalHeader()->hide();
    m_fakturyPrijate->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(m_fakturyPrijate, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(slotDoubleclickFaktura(QModelIndex)));
    connect(m_fakturyPrijate, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(slotPopup(QPoint)));
    QVBoxLayout *mainFrame = new QVBoxLayout;
    mainFrame->addWidget(faktura);
    mainFrame->addWidget(m_fakturyPrijate);
    m_panel->setLayout(mainFrame);
}

QWidget *Prijate::getPanel()
{
    return m_panel;
}

void Prijate::firmaChanged()
{
    m_modelPrijate->setQuery(QString("SELECT * FROM faktura_view WHERE firmaid=%1 AND vydana=0 AND rok=%2")
                             .arg(preferences.m_firmaid)
                             .arg(preferences.m_rok));
    if(!m_modelPrijate->rowCount())
    {
        m_editPrijata->setEnabled(false);
        m_deletePrijata->setEnabled(false);
        m_printPrijata->setEnabled(false);
        m_payPrijata->setEnabled(false);
    }
    else
    {
        m_editPrijata->setEnabled(true);
        m_deletePrijata->setEnabled(true);
        m_printPrijata->setEnabled(true);
        m_payPrijata->setEnabled(true);
    }
    if(dxutils::isPlatce())
        m_fakturyPrijate->showColumn(8);
    else
        m_fakturyPrijate->hideColumn(8);
}

void Prijate::createActions()
{
    m_addPrijata = new QAction(QIcon(":/icons/add.png"), QString::fromUtf8("Přijmout"), this);
    m_addPrijata->setToolTip(QString::fromUtf8("Přijmout fakturu"));
    m_addPrijata->setStatusTip(QString::fromUtf8("Přijmout fakturu"));
    connect(m_addPrijata, SIGNAL(triggered()), this, SLOT(slotCmdAddFaktura()));

    m_editPrijata = new QAction(QIcon(":/icons/edit.png"), "Upravit", this);
    m_editPrijata->setToolTip("Upravit fakturu");
    m_editPrijata->setStatusTip("Upravit fakturu");
    connect(m_editPrijata, SIGNAL(triggered()), this, SLOT(slotCmdEditFaktura()));
    if(!m_modelPrijate->rowCount()) m_editPrijata->setEnabled(false);

    m_deletePrijata = new QAction(QIcon(":/icons/delete.png"), "Smazat", this);
    m_deletePrijata->setToolTip("Smazat fakturu");
    m_deletePrijata->setStatusTip("Smazat fakturu");
    connect(m_deletePrijata, SIGNAL(triggered()), this, SLOT(slotCmdDeleteFaktura()));
    if(!m_modelPrijate->rowCount()) m_deletePrijata->setEnabled(false);

    m_printPrijata = new QAction(QIcon(":/icons/print.png"), "Tisk", this);
    m_printPrijata->setToolTip("Vytisknout fakturu");
    m_printPrijata->setStatusTip("Vytisknout fakturu");
    connect(m_printPrijata, SIGNAL(triggered()), this, SLOT(slotCmdPrintFaktura()));
    if(!m_modelPrijate->rowCount()) m_printPrijata->setEnabled(false);
    
    m_payPrijata = new QAction(QIcon(":/icons/money.png"), QString::fromUtf8("Úhrada"), this);
    m_payPrijata->setToolTip(QString::fromUtf8("Zaúčtovat úhradu"));
    m_payPrijata->setStatusTip(QString::fromUtf8("Zaúčtovat úhradu"));
    connect(m_payPrijata, SIGNAL(triggered()), this, SLOT(slotCmdPayFaktura()));
    if(!m_modelPrijate->rowCount()) m_payPrijata->setEnabled(false);
}

void Prijate::setupModel()
{
    m_modelPrijate = new FaModel(this);
    m_modelPrijate->setQuery(QString("SELECT * FROM faktura_view WHERE firmaid=%1 AND vydana=0 AND rok=%2")
                             .arg(preferences.m_firmaid)
                             .arg(preferences.m_rok));
    m_modelPrijate->setHeaderData(1, Qt::Horizontal, "Typ");
    m_modelPrijate->setHeaderData(2, Qt::Horizontal, QString::fromUtf8("Číslo"));
    m_modelPrijate->setHeaderData(3, Qt::Horizontal, "Popis");
    m_modelPrijate->setHeaderData(4, Qt::Horizontal, "Dodavatel");
    m_modelPrijate->setHeaderData(5, Qt::Horizontal, "Celkem");
    m_modelPrijate->setHeaderData(6, Qt::Horizontal, QString::fromUtf8("Měna"));
    m_modelPrijate->setHeaderData(7, Qt::Horizontal, "Vystavena");
    m_modelPrijate->setHeaderData(8, Qt::Horizontal, "DUZP");
    m_modelPrijate->setHeaderData(9, Qt::Horizontal, QString::fromUtf8("Splatná"));
    m_modelPrijate->setHeaderData(10, Qt::Horizontal, "Zaplacena");
}

void Prijate::slotCmdAddFaktura()
{
    FatypDialog *typ = new FatypDialog(m_panel);
    typ->setModal(true);
    if(typ->exec() == QDialog::Accepted)
    {
        FakturaDialog *dialog = new FakturaDialog(typ->typ(), false, m_panel);
        dialog->setModal(true);
        if(dialog->exec() == QDialog::Accepted)
        {
            m_modelPrijate->setQuery(QString("SELECT * FROM faktura_view WHERE firmaid=%1 AND vydana=0 AND rok=%2")
                                     .arg(preferences.m_firmaid)
                                     .arg(preferences.m_rok));
            m_editPrijata->setEnabled(true);
            m_deletePrijata->setEnabled(true);
            m_printPrijata->setEnabled(true);
            m_payPrijata->setEnabled(true);
        }
    }
}

void Prijate::slotCmdDeleteFaktura()
{
    QModelIndex current = m_fakturyPrijate->currentIndex();
    if(QMessageBox::question(m_panel, "Dotaz", QString::fromUtf8("Opravdu smazat %1 \nčíslo %2?")
        .arg(m_modelPrijate->record(current.isValid()?current.row():0).value("typ").toString())
        .arg(m_modelPrijate->record(current.isValid()?current.row():0).value("cislo").toString()),
        QMessageBox::Ok|QMessageBox::Cancel) == QMessageBox::Ok)
    {
        int id = current.isValid() ? m_modelPrijate->record(current.row()).value("id").toInt() :
                m_modelPrijate->record(0).value("id").toInt();
        QSqlQuery q;
        q.exec(QString("DELETE FROM faktura WHERE id=%1").arg(id));
        m_modelPrijate->setQuery(QString("SELECT * FROM faktura_view WHERE firmaid=%1 AND vydana=0 AND rok=%2")
                                 .arg(preferences.m_firmaid)
                                 .arg(preferences.m_rok));
        if(!m_modelPrijate->rowCount())
        {
            m_editPrijata->setEnabled(false);
            m_deletePrijata->setEnabled(false);
            m_printPrijata->setEnabled(false);
            m_payPrijata->setEnabled(false);
        }
    }
}

void Prijate::slotCmdEditFaktura()
{
    QModelIndex current = m_fakturyPrijate->currentIndex();
    int id;
    if(current.isValid())
        id = m_modelPrijate->record(current.row()).value("id").toInt();
    else
        id = m_modelPrijate->record(0).value("id").toInt();
    FakturaDialog *dialog = new FakturaDialog(id, m_panel);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        m_modelPrijate->setQuery(QString("SELECT * FROM faktura_view WHERE firmaid=%1 AND vydana=0 AND rok=%2")
                                 .arg(preferences.m_firmaid)
                                 .arg(preferences.m_rok));
    }
}

void Prijate::slotCmdPrintFaktura()
{
    QModelIndex current = m_fakturyPrijate->currentIndex();
    int id;
    if(current.isValid())
        id = m_modelPrijate->record(current.row()).value("id").toInt();
    else
        id = m_modelPrijate->record(0).value("id").toInt();
    PrintfaDialog *dialog = new PrintfaDialog(id, m_panel);
    dialog->setModal(true);
    dialog->show();
}

void Prijate::slotCmdPayFaktura()
{
    QModelIndex current = m_fakturyPrijate->currentIndex();
    int id;
    QString cislo, zaplaceno;
    if(current.isValid())
    {
        id = m_modelPrijate->record(current.row()).value("id").toInt();
        cislo = m_modelPrijate->record(current.row()).value("cislo").toString();
        zaplaceno = m_modelPrijate->record(current.row()).value("zaplaceno").toString();
    }
    else
    {
        id = m_modelPrijate->record(0).value("id").toInt();
        cislo = m_modelPrijate->record(0).value("cislo").toString();
        zaplaceno = m_modelPrijate->record(0).value("zaplaceno").toString();
    }
    UhradaDialog *dialog = new UhradaDialog(cislo
        ,zaplaceno.isEmpty()?QDate::currentDate():QDate::fromString(zaplaceno, "dd.MM.yyyy")
        , m_panel);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        QSqlQuery q;
        q.prepare("UPDATE faktura SET zaplaceno=? WHERE id=?");
        q.addBindValue(dialog->datum());
        q.addBindValue(id);
        if(!q.exec())
            QMessageBox::warning(m_panel, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
        m_modelPrijate->setQuery(QString("SELECT * FROM faktura_view WHERE firmaid=%1 AND vydana=0 AND rok=%2")
                                 .arg(preferences.m_firmaid)
                                 .arg(preferences.m_rok));
    }
}

void Prijate::slotDoubleclickFaktura(const QModelIndex &index)
{
    FakturaDialog *dialog = new FakturaDialog(m_modelPrijate->record(index.row()).value("id").toInt(), m_panel);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        m_modelPrijate->setQuery(QString("SELECT * FROM faktura_view WHERE firmaid=%1 AND vydana=0 AND rok=%2")
                                 .arg(preferences.m_firmaid)
                                 .arg(preferences.m_rok));
    }
}

void Prijate::slotPopup(const QPoint &pos)
{
    QModelIndex current = m_fakturyPrijate->currentIndex();
    if(current.isValid())
    {
        QMenu popup;
        popup.addAction(m_editPrijata);
        popup.addAction(m_deletePrijata);
        popup.addSeparator();
        popup.addAction(m_printPrijata);
        popup.addSeparator();
        popup.addAction(m_payPrijata);
        popup.exec(m_fakturyPrijate->mapToGlobal(pos));
    }
}

