/***************************************************************
 * Name:      vydane.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include <QToolBar>
#include "vydane.h"
#include "dxutils.h"
#include "dialogs/fatypdialog.h"
#include "dialogs/fakturadialog.h"
#include "dialogs/printfadialog.h"
#include "dialogs/uhradadialog.h"
#include "dialogs/emailfadialog.h"
#include "preferences.h"

Vydane::Vydane(QTreeWidgetItem *parent) :
    dxitem(parent)
{
    setupModel();
    createActions();
    m_panel = new QWidget;
    QToolBar *faktura = new QToolBar(m_panel);
    faktura->setStyleSheet("QToolBar { border: 0px }");
    faktura->setMovable(false);
    faktura->addAction(m_addVydana);
    faktura->addAction(m_editVydana);
    faktura->addAction(m_deleteVydana);
    faktura->addSeparator();
    faktura->addAction(m_printVydana);
    faktura->addAction(m_emailVydana);
    faktura->addSeparator();
    faktura->addAction(m_payVydana);
    m_fakturyVydane = new QTableView;
    m_fakturyVydane->setModel(m_modelVydane);
    m_fakturyVydane->hideColumn(0);
    if(!dxutils::isPlatce()) m_fakturyVydane->hideColumn(8);
    m_fakturyVydane->hideColumn(11);
    m_fakturyVydane->hideColumn(12);
    m_fakturyVydane->hideColumn(13);
    m_fakturyVydane->horizontalHeader()->setStretchLastSection(true);
    m_fakturyVydane->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_fakturyVydane->setSelectionMode(QAbstractItemView::SingleSelection);
    m_fakturyVydane->verticalHeader()->hide();
    m_fakturyVydane->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(m_fakturyVydane, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(slotDoubleclickFaktura(QModelIndex)));
    connect(m_fakturyVydane, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(slotPopup(QPoint)));
    QVBoxLayout *mainFrame = new QVBoxLayout;
    mainFrame->addWidget(faktura);
    mainFrame->addWidget(m_fakturyVydane);
    m_panel->setLayout(mainFrame);
}

QWidget *Vydane::getPanel()
{
    return m_panel;
}

void Vydane::firmaChanged()
{
    m_modelVydane->setQuery(QString("SELECT * FROM faktura_view WHERE firmaid=%1 AND vydana=1 AND rok=%2")
                            .arg(preferences.m_firmaid)
                            .arg(preferences.m_rok));
    if(!m_modelVydane->rowCount())
    {
        m_editVydana->setEnabled(false);
        m_deleteVydana->setEnabled(false);
        m_printVydana->setEnabled(false);
        m_emailVydana->setEnabled(false);
        m_payVydana->setEnabled(false);
    }
    else
    {
        m_editVydana->setEnabled(true);
        m_deleteVydana->setEnabled(true);
        m_printVydana->setEnabled(true);
        m_emailVydana->setEnabled(true);
        m_payVydana->setEnabled(true);
    }
    if(dxutils::isPlatce())
        m_fakturyVydane->showColumn(8);
    else
        m_fakturyVydane->hideColumn(8);
}

void Vydane::createActions()
{
    m_addVydana = new QAction(QIcon(":/icons/add.png"), QString::fromUtf8("Vydat"), this);
    m_addVydana->setToolTip(QString::fromUtf8("Vydat fakturu"));
    m_addVydana->setStatusTip(QString::fromUtf8("Vydat fakturu"));
    connect(m_addVydana, SIGNAL(triggered()), this, SLOT(slotCmdAddFaktura()));

    m_editVydana = new QAction(QIcon(":/icons/edit.png"), "Upravit", this);
    m_editVydana->setToolTip("Upravit fakturu");
    m_editVydana->setStatusTip("Upravit fakturu");
    connect(m_editVydana, SIGNAL(triggered()), this, SLOT(slotCmdEditFaktura()));
    if(!m_modelVydane->rowCount()) m_editVydana->setEnabled(false);

    m_deleteVydana = new QAction(QIcon(":/icons/delete.png"), "Smazat", this);
    m_deleteVydana->setToolTip("Smazat fakturu");
    m_deleteVydana->setStatusTip("Smazat fakturu");
    connect(m_deleteVydana, SIGNAL(triggered()), this, SLOT(slotCmdDeleteFaktura()));
    if(!m_modelVydane->rowCount()) m_deleteVydana->setEnabled(false);

    m_printVydana = new QAction(QIcon(":/icons/print.png"), "Tisk", this);
    m_printVydana->setToolTip("Vytisknout fakturu");
    m_printVydana->setStatusTip("Vytisknout fakturu");
    connect(m_printVydana, SIGNAL(triggered()), this, SLOT(slotCmdPrintFaktura()));
    if(!m_modelVydane->rowCount()) m_printVydana->setEnabled(false);

    m_emailVydana = new QAction(QIcon(":/icons/email.png"), "Email", this);
    m_emailVydana->setToolTip("Poslat fakturu emailem");
    m_emailVydana->setStatusTip("Poslat fakturu emailem");
    connect(m_emailVydana, SIGNAL(triggered()), this, SLOT(slotCmdEmailFaktura()));
    if(!m_modelVydane->rowCount()) m_emailVydana->setEnabled(false);
    
    m_payVydana = new QAction(QIcon(":/icons/money.png"), QString::fromUtf8("Úhrada"), this);
    m_payVydana->setToolTip(QString::fromUtf8("Zaúčtovat úhradu"));
    m_payVydana->setStatusTip(QString::fromUtf8("Zaúčtovat úhradu"));
    connect(m_payVydana, SIGNAL(triggered()), this, SLOT(slotCmdPayFaktura()));
    if(!m_modelVydane->rowCount()) m_payVydana->setEnabled(false);
}

void Vydane::setupModel()
{
    m_modelVydane = new FaModel(this);
    m_modelVydane->setQuery(QString("SELECT * FROM faktura_view WHERE firmaid=%1 AND vydana=1 AND rok=%2")
                            .arg(preferences.m_firmaid)
                            .arg(preferences.m_rok));
    m_modelVydane->setHeaderData(1, Qt::Horizontal, "Typ");
    m_modelVydane->setHeaderData(2, Qt::Horizontal, QString::fromUtf8("Číslo"));
    m_modelVydane->setHeaderData(3, Qt::Horizontal, "Popis");
    m_modelVydane->setHeaderData(4, Qt::Horizontal, QString::fromUtf8("Odběratel"));
    m_modelVydane->setHeaderData(5, Qt::Horizontal, "Celkem");
    m_modelVydane->setHeaderData(6, Qt::Horizontal, QString::fromUtf8("Měna"));
    m_modelVydane->setHeaderData(7, Qt::Horizontal, "Vystavena");
    m_modelVydane->setHeaderData(8, Qt::Horizontal, "DUZP");
    m_modelVydane->setHeaderData(9, Qt::Horizontal, QString::fromUtf8("Splatná"));
    m_modelVydane->setHeaderData(10, Qt::Horizontal, "Zaplacena");
}

void Vydane::slotCmdAddFaktura()
{
    FatypDialog *typ = new FatypDialog(m_panel);
    typ->setModal(true);
    if(typ->exec() == QDialog::Accepted)
    {
        FakturaDialog *dialog = new FakturaDialog(typ->typ(), true, m_panel);
        dialog->setModal(true);
        if(dialog->exec() == QDialog::Accepted)
        {
            m_modelVydane->setQuery(QString("SELECT * FROM faktura_view WHERE firmaid=%1 AND vydana=1 AND rok=%2")
                                    .arg(preferences.m_firmaid)
                                    .arg(preferences.m_rok));
            m_editVydana->setEnabled(true);
            m_deleteVydana->setEnabled(true);
            m_printVydana->setEnabled(true);
            m_emailVydana->setEnabled(true);
            m_payVydana->setEnabled(true);
        }
    }
}

void Vydane::slotCmdDeleteFaktura()
{
    QModelIndex current = m_fakturyVydane->currentIndex();
    if(QMessageBox::question(m_panel, "Dotaz", QString::fromUtf8("Opravdu smazat %1 \nčíslo %2?")
        .arg(m_modelVydane->record(current.isValid()?current.row():0).value("typ").toString())
        .arg(m_modelVydane->record(current.isValid()?current.row():0).value("cislo").toString()),
        QMessageBox::Ok|QMessageBox::Cancel) == QMessageBox::Ok)
    {
        int id = current.isValid() ? m_modelVydane->record(current.row()).value("id").toInt() :
                m_modelVydane->record(0).value("id").toInt();
        QSqlQuery q;
        q.exec(QString("DELETE FROM faktura WHERE id=%1").arg(id));
        m_modelVydane->setQuery(QString("SELECT * FROM faktura_view WHERE firmaid=%1 AND vydana=1 AND rok=%2")
                                .arg(preferences.m_firmaid)
                                .arg(preferences.m_rok));
        if(!m_modelVydane->rowCount())
        {
            m_editVydana->setEnabled(false);
            m_deleteVydana->setEnabled(false);
            m_printVydana->setEnabled(false);
            m_emailVydana->setEnabled(false);
            m_payVydana->setEnabled(false);
        }
    }
}

void Vydane::slotCmdEditFaktura()
{
    QModelIndex current = m_fakturyVydane->currentIndex();
    int id;
    if(current.isValid())
        id = m_modelVydane->record(current.row()).value("id").toInt();
    else
        id = m_modelVydane->record(0).value("id").toInt();
    FakturaDialog *dialog = new FakturaDialog(id, m_panel);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        m_modelVydane->setQuery(QString("SELECT * FROM faktura_view WHERE firmaid=%1 AND vydana=1 AND rok=%2")
                                .arg(preferences.m_firmaid)
                                .arg(preferences.m_rok));
    }
}

void Vydane::slotCmdPrintFaktura()
{
    QModelIndex current = m_fakturyVydane->currentIndex();
    int id;
    if(current.isValid())
        id = m_modelVydane->record(current.row()).value("id").toInt();
    else
        id = m_modelVydane->record(0).value("id").toInt();
    PrintfaDialog *dialog = new PrintfaDialog(id, m_panel);
    dialog->setModal(true);
    dialog->show();
}

void Vydane::slotCmdEmailFaktura()
{
    QModelIndex current = m_fakturyVydane->currentIndex();
    int id;
    if(current.isValid())
        id = m_modelVydane->record(current.row()).value("id").toInt();
    else
        id = m_modelVydane->record(0).value("id").toInt();
    EmailfaDialog *dialog = new EmailfaDialog(id, m_panel);
    dialog->setModal(true);
    dialog->show();
}

void Vydane::slotCmdPayFaktura()
{
    QModelIndex current = m_fakturyVydane->currentIndex();
    int id;
    QString cislo, zaplaceno;
    if(current.isValid())
    {
        id = m_modelVydane->record(current.row()).value("id").toInt();
        cislo = m_modelVydane->record(current.row()).value("cislo").toString();
        zaplaceno = m_modelVydane->record(current.row()).value("zaplaceno").toString();
    }
    else
    {
        id = m_modelVydane->record(0).value("id").toInt();
        cislo = m_modelVydane->record(0).value("cislo").toString();
        zaplaceno = m_modelVydane->record(0).value("zaplaceno").toString();
    }
    UhradaDialog *dialog = new UhradaDialog(cislo
        ,zaplaceno.isEmpty()?QDate::currentDate():QDate::fromString(zaplaceno, "dd.MM.yyyy")
        , m_panel);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        QSqlQuery q;
        q.prepare("UPDATE faktura SET zaplaceno=? WHERE id=?");
        q.addBindValue(dialog->datum());
        q.addBindValue(id);
        if(!q.exec())
            QMessageBox::warning(m_panel, "dxfaktura", QString::fromUtf8("Databáze reportovala chybu: %1").arg(q.lastError().text()));
        m_modelVydane->setQuery(QString("SELECT * FROM faktura_view WHERE firmaid=%1 AND vydana=1 AND rok=%2")
                                .arg(preferences.m_firmaid)
                                .arg(preferences.m_rok));
    }
}


void Vydane::slotDoubleclickFaktura(const QModelIndex &index)
{
    FakturaDialog *dialog = new FakturaDialog(m_modelVydane->record(index.row()).value("id").toInt(), m_panel);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        m_modelVydane->setQuery(QString("SELECT * FROM faktura_view WHERE firmaid=%1 AND vydana=1 AND rok=%2")
                                .arg(preferences.m_firmaid)
                                .arg(preferences.m_rok));
    }
}

void Vydane::slotPopup(const QPoint &pos)
{
    QModelIndex current = m_fakturyVydane->currentIndex();
    if(current.isValid())
    {
        QMenu popup;
        popup.addAction(m_editVydana);
        popup.addAction(m_deleteVydana);
        popup.addSeparator();
        popup.addAction(m_printVydana);
        popup.addAction(m_emailVydana);
        popup.addSeparator();
        popup.addAction(m_payVydana);
        popup.exec(m_fakturyVydane->mapToGlobal(pos));
    }
}

