/***************************************************************
 * Name:      preferences.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include "preferences.h"
#include <QSettings>
#include <QApplication>
#include <QDesktopServices>
#include <QProcessEnvironment>
#include <QDir>
#include <QDate>

Preferences::Preferences()
{
    //Fakturace settings value
    m_rok = QDate::currentDate().toString("yyyy").toInt();
    m_prefixVydana = QString::number(m_rok);
    m_prefixPrijata = m_prefixVydana;
    m_splatnost = 14;
    m_faprijataneplatce = "internal";
    m_faprijataplatce = "internal";
    m_favydananeplatce = "internal";
    m_favydanaplatce = "internal";
    m_zalohaprijataneplatce = "internal";
    m_zalohaprijataplatce = "internal";
    m_zalohavydananeplatce = "internal";
    m_zalohavydanaplatce = "internal";
    //SMTP settings value
    m_server = QString();
    m_port = 25;
    m_ssl = false;
    m_autentifikace = false;
    m_username = QString();
    m_password = QString();
    m_sender = "Jan Novak <novak@novakov.cz>";
    m_subject = "Faktura";
    m_text = QString::fromUtf8("Dobrý den,\nv příloze Vám zasíláme fakturu");
    //Other settings value
    m_x = 50;
    m_y = 50;
    m_w = 888;
    m_h = 666;
    m_maximized = false;
    m_firmaid = 1;

    QString iniPath;
#ifdef Q_OS_WIN
    iniPath = QProcessEnvironment::systemEnvironment().value("APPDATA")+"/dxfaktura";
#else
#if (QT_VERSION > 0x050000)
    iniPath = QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0)+"/.config/dxfaktura";
#else
    iniPath = QDesktopServices::storageLocation(QDesktopServices::HomeLocation)+"/.config/dxfaktura";
#endif
#endif
    QDir dir;
    if(!dir.exists(iniPath))
        dir.mkpath(iniPath);
    m_iniFile = iniPath+"/dxfaktura.ini";
}

Preferences &Preferences::instance()
{
    static Preferences p;
    return p;
}

void Preferences::load()
{
    QSettings set(m_iniFile, QSettings::IniFormat);
    //Fakturace settings value
    set.beginGroup("FAKTURACE");
    m_rok = set.value("rok", m_rok).toInt();
    m_prefixVydana = set.value("prefixVydana", m_prefixVydana).toString();
    m_prefixPrijata = set.value("prefixPrijata", m_prefixPrijata).toString();
    m_splatnost = set.value("splatnost", m_splatnost).toInt();
    m_faprijataneplatce = validTemplate(set.value("faprijataneplatce", m_faprijataneplatce).toString());
    m_faprijataplatce = validTemplate(set.value("faprijataplatce", m_faprijataplatce).toString());
    m_favydananeplatce = validTemplate(set.value("favydananeplatce", m_favydananeplatce).toString());
    m_favydanaplatce = validTemplate(set.value("favydanaplatce", m_favydanaplatce).toString());
    m_zalohaprijataneplatce = validTemplate(set.value("zalohaprijataneplatce", m_zalohaprijataneplatce).toString());
    m_zalohaprijataplatce = validTemplate(set.value("zalohaprijataplatce", m_zalohaprijataplatce).toString());
    m_zalohavydananeplatce = validTemplate(set.value("zalohavydananeplatce", m_zalohavydananeplatce).toString());
    m_zalohavydanaplatce = validTemplate(set.value("zalohavydanaplatce", m_zalohavydanaplatce).toString());
    set.endGroup();
    //SMTP settings value
    set.beginGroup("SMTP");
    m_server = set.value("server", m_server).toString();
    m_port = set.value("port", m_port).toInt();
    m_ssl = set.value("ssl", m_ssl).toBool();
    m_autentifikace = set.value("autentifikace", m_autentifikace).toBool();
    m_username = set.value("username", m_username).toString();
    m_password = set.value("password", m_password).toString();
    m_sender = set.value("sender", m_sender).toString();
    m_subject = set.value("subject", m_subject).toString();
    m_text = set.value("text", m_text).toString();
    set.endGroup();
    //Other settings value
    set.beginGroup("OTHER");
    m_x = set.value("x", m_x).toInt();
    m_y = set.value("y", m_y).toInt();
    m_w = set.value("w", m_w).toInt();
    m_h = set.value("h", m_h).toInt();
    m_maximized = set.value("maximized", m_maximized).toBool();
    m_firmaid = set.value("firmaid", m_firmaid).toInt();
    set.endGroup();
}

void Preferences::save()
{
    QSettings set(m_iniFile, QSettings::IniFormat);
    foreach(QString key, set.allKeys())
        set.remove(key);
    //Fakturace settings value
    set.beginGroup("FAKTURACE");
    set.setValue("rok", m_rok);
    set.setValue("prefixVydana", m_prefixVydana);
    set.setValue("prefixPrijata", m_prefixPrijata);
    set.setValue("splatnost", m_splatnost);
    set.setValue("faprijataneplatce", m_faprijataneplatce);
    set.setValue("faprijataplatce", m_faprijataplatce);
    set.setValue("favydananeplatce", m_favydananeplatce);
    set.setValue("favydanaplatce", m_favydanaplatce);
    set.setValue("zalohaprijataneplatce", m_zalohaprijataneplatce);
    set.setValue("zalohaprijataplatce", m_zalohaprijataplatce);
    set.setValue("zalohavydananeplatce", m_zalohavydananeplatce);
    set.setValue("zalohavydanaplatce", m_zalohavydanaplatce);
    set.endGroup();
    //SMTP settings value
    set.beginGroup("SMTP");
    set.setValue("server", m_server);
    set.setValue("port", m_port);
    set.setValue("ssl", m_ssl);
    set.setValue("autentifikace", m_autentifikace);
    set.setValue("username", m_username);
    set.setValue("password", m_password);
    set.setValue("sender", m_sender);
    set.setValue("subject", m_subject);
    set.setValue("text", m_text);
    set.endGroup();
    //Other settings value
    set.beginGroup("OTHER");
    set.setValue("x", m_x);
    set.setValue("y", m_y);
    set.setValue("w", m_w);
    set.setValue("h", m_h);
    set.setValue("maximized", m_maximized);
    set.setValue("firmaid", m_firmaid);
    set.endGroup();
    set.sync();
}

QString Preferences::validTemplate(const QString &name)
{
    if(name.isEmpty())
        return "internal";
    if(name == "internal")
        return name;
    QDir dir;
    if(dir.exists(name))
        return name;
    else
        return "internal";
}
