/***************************************************************
 * Name:      helpbrowser.cpp
 * Author:    Lubos Dolezel <lubos a dolezel.info>
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <QModelIndex>
#include <QItemSelectionModel>
#include <QHelpContentWidget>
#include <QDesktopServices>
#include "helpbrowser.h"

HelpBrowser::HelpBrowser(QWidget *parent) :
    QTextBrowser(parent), m_engine(0)
{
}

void HelpBrowser::setEngine(QHelpEngine *engine)
{
    m_engine = engine;
}

QVariant HelpBrowser::loadResource(int type, const QUrl& name)
{
    QUrl url(name);
    if(name.scheme() == "http")
    {
        QMetaObject::invokeMethod(this, "setSource", Qt::QueuedConnection, Q_ARG(QUrl, m_urlLast));
        QDesktopServices::openUrl(name);
        return QVariant();
    }
    if(!m_engine)
        return QVariant();
    if(type == QTextDocument::HtmlResource)
    {
        QModelIndex index;
        QHelpContentWidget* w = m_engine->contentWidget();
        QItemSelectionModel* selmodel;
        m_urlLast = name;
        index = w->indexOf(m_urlLast);
        w->blockSignals(true);
        selmodel = w->selectionModel();
        selmodel->clearSelection();
        selmodel->select(index, QItemSelectionModel::Select);
        w->blockSignals(false);
    }
    if(url.isRelative())
        url = source().resolved(url);
    return m_engine->fileData(url);
}
